# -*- coding: utf-8 -*-

from imio.smartweb.common.contact_utils import ContactProperties as ContactSchedule
from imio.smartweb.common.utils import rich_description
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.utils import hash_md5
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.i18n import translate

import json


class ContactProperties(ContactSchedule):
    def __init__(self, json_dict, section):
        self.contact = json_dict
        self.context = section

    def __getattr__(self, name):
        return self.contact.get(name)

    @property
    def contact_type_class(self):
        contact_type = self.contact.get("type").get("token")
        return "contact-type-{}".format(contact_type)

    @property
    def description(self):
        description = rich_description(self.contact.get("description"))
        return description

    def logo(self):
        if self.contact.get("logo") is None:
            return ""
        modified_hash = hash_md5(self.contact["modified"])
        logo = f"{self.contact['@id']}/@@images/logo/preview?cache_key={modified_hash}"
        return logo

    def leadimage(self):
        if self.contact.get("image") is None:
            return ""
        modified_hash = hash_md5(self.contact["modified"])
        leadimage = f"{self.contact['@id']}/@@images/image/{self.context.orientation}_affiche?cache_key={modified_hash}"
        return leadimage

    def data_geojson(self):
        """Return the contact geolocation as GeoJSON string."""
        current_lang = api.portal.get_current_language()[:2]
        coordinates = self.contact.get("geolocation")
        longitude = coordinates.get("longitude")
        latitude = coordinates.get("latitude")
        link_text = translate(_("Itinerary"), target_language=current_lang)
        geo_json = {
            "type": "Feature",
            "properties": {
                "popup": '<a href="{}">{}</a>'.format(
                    self.get_itinerary_link(), link_text
                ),
            },
            "geometry": {
                "type": "Point",
                "coordinates": [
                    longitude,
                    latitude,
                ],
            },
        }
        return json.dumps(geo_json)

    def images(self, image_scale, nb_results_by_batch):
        contact_url = self.contact["@id"]
        query = "@search?portal_type=Image&path.depth=1&metadata_fields=modified"
        images_url_request = "{}/{}".format(contact_url, query)
        json_images = get_json(images_url_request)
        if json_images is None or len(json_images.get("items", [])) == 0:
            return
        results = []
        thumb_scale = image_scale
        for image in json_images.get("items"):
            base_url = image["@id"]
            modified_hash = hash_md5(image["modified"])
            large_url = f"{base_url}/@@images/image/?cache_key={modified_hash}"
            url = f"{base_url}/@@images/image/paysage_{thumb_scale}?cache_key={modified_hash}"
            dict_item = {
                "title": image["title"],
                "description": image["description"],
                "image_large_url": large_url,
                "image_url": url,
            }
            results.append(dict_item)
        return batch_results(results, nb_results_by_batch)

    def get_itinerary_link(self):
        address_parts = [
            self.contact.get("street"),
            self.contact.get("number") and str(self.contact.get("number")) or "",
            self.contact.get("complement"),
            self.contact.get("zipcode") and str(self.contact.get("zipcode")) or "",
            self.contact.get("city"),
        ]
        if self.contact.get("country"):
            address_parts.append(self.contact.get("country").get("title"))
        address = "+".join(filter(None, address_parts))
        if not address:
            return
        return "https://www.google.com/maps/dir/?api=1&destination={}".format(address)

    def get_translated_url_type(self, url_type_id):
        current_lang = api.portal.get_current_language()[:2]
        url_type_label = url_type_id[0].upper() + url_type_id[1:]
        return translate(_(url_type_label), target_language=current_lang)

    def formatted_address(self):
        street_parts = [
            self.contact.get("street"),
            self.contact.get("number") and str(self.contact.get("number")) or "",
            self.contact.get("complement"),
        ]
        street = " ".join(filter(None, street_parts))
        entity_parts = [
            self.contact.get("zipcode") and str(self.contact.get("zipcode")) or "",
            self.contact.get("city"),
        ]
        entity = " ".join(filter(None, entity_parts))
        country = (
            self.contact.get("country")
            and self.contact.get("country").get("title")
            or ""
        )
        if not (street or entity or country):
            return None
        return {"street": street, "entity": entity, "country": country}



# -*- coding: utf-8 -*-

from embeddify import Embedder
from embeddify import Plugin

from imio.smartweb.core.interfaces import IOdwbViewUtils
from imio.smartweb.core.contents.sections.views import SectionView
from imio.smartweb.core.interfaces import IArcgisViewUtils
from imio.smartweb.locales import SmartwebMessageFactory as _
from json.decoder import JSONDecodeError
from plone import api
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from Products.Five.browser import BrowserView
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import json
import logging
import requests

logger = logging.getLogger("imio.smartweb.core")


class ExternalContentView(SectionView):

    plugin_config = {}

    def __init__(self, context, request):
        width = "100%"
        current_lang = api.portal.get_current_language()[:2]
        plugins = [
            EaglebePlugin(),
            EllohaPlugin(),
            CognitoformPlugin(),
            ArcgisPlugin(),
            GiveADayPlugin(),
            IdeluxWastePlugin(),
            InbwContainersAffluencePlugin(),
            OdwbWidgetPlugin(),
            UnknowServicePlugin(),
        ]
        extra_params = context.external_content_params
        self.plugin_config = {
            "eaglebeplugin": {"width": width},
            "ellohaplugin": {
                "width": width,
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "cognitoformplugin": {
                "width": width,
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "arcgisplugin": {
                "width": width,
                "url": context.absolute_url(),
                "current_lang": current_lang,
                "extra_params": extra_params,
            },
            "giveadayplugin": {},
            "ideluxwasteplugin": {
                "url": context.absolute_url(),
                "extra_params": extra_params,
            },
            "inbwcontainersaffluenceplugin": {
                "url": context.absolute_url(),
                "extra_params": extra_params,
            },
            "odwbwidgetplugin": {
                "url": context.absolute_url(),
                "extra_params": extra_params,
            },
            "unknowserviceplugin": {"width": width, "current_lang": current_lang},
        }
        embedder = Embedder(plugins=plugins, plugin_config=self.plugin_config)
        url = context.external_content_url
        self.plugin = embedder(url, config={"width": width})
        super(ExternalContentView, self).__init__(context, request)

    @property
    def contents(self):
        return self.plugin.contents

    def display_odwb_widget_viewlet(self):
        return self.plugin.is_odwb_widget_plugins

    def render_viewlet(self):
        return self.index()

    def has_leadimage(self):
        if ILeadImage.providedBy(self.context) and getattr(
            self.context, "image", False
        ):
            return True
        return False

    def get_plugin(self):
        return self.plugin

    def which_plugin(self):
        if IOdwbWidgetPlugin.providedBy(self.plugin):
            return "odwbwidgetplugin"
        elif IIdeluxWastePlugin.providedBy(self.plugin):
            return "ideluxwasteplugin"
        elif IInbwContainersAffluencePlugin.providedBy(self.plugin):
            return "inbwcontainersaffluenceplugin"
        else:
            return None

    def image(self):
        return f"{self.context.absolute_url()}/@@download/image"


class BasePlugin(Plugin):
    """ """

    is_odwb_widget_plugins = False

    @property
    def contents(self):
        return ""


class EaglebePlugin(BasePlugin):
    def __call__(self, parts, config={}):
        self.parts = parts
        self.config = config
        if "app.eaglebe.com" in parts.netloc:
            return self
        return None

    @property
    def contents(self):
        return f'<iframe class="eaglebe" src="{self.parts.geturl()}" scrolling="no" width="{self.config["width"]}">'


class EllohaPlugin(BasePlugin):
    def __call__(self, parts, config={}):
        if "reservation.elloha.com" in parts.netloc:
            self.parts = parts
            self.config = config
            return self
        return None

    @property
    def contents(self):
        # sample :
        # {
        #     "ConstellationWidgetContainer": "917c4c52-c997-4077-9135-873a0b2e0c85",
        #     "Idoi": "195ea849-1732-4a69-a051-e7911001cd09"
        # }
        current_lang = self.config["current_lang"]
        extra_params = self.config["extra_params"]
        error_message = _(
            "With an elloha plugin, extra params must contain a dictionary with two keys : ConstellationWidgetContainer, Idoi"
        )
        error_message = translate(error_message, target_language=current_lang)
        error_message = f'<div class="elloha elloha_error">{error_message}</div>'

        if extra_params is None or extra_params[0] != "{" or extra_params[-1] != "}":
            return error_message
        try:
            res = json.loads(extra_params.lower())
        except JSONDecodeError:
            return error_message

        if res.get("constellationwidgetcontainer") is None or res.get("idoi") is None:
            return error_message

        cw1 = res.get("constellationwidgetcontainer")
        cw2 = cw1.replace("-", "")
        idoi = res.get("idoi")
        return (
            f'<div class="elloha">'
            f'<div id="ConstellationWidgetContainer{cw1}" '
            f'style="width:100%" data-id-projet="{cw2}">...</div></div>'
            f'<script type="text/javascript" src="{self.parts.geturl()}/Scripts/widget-loader.min.js?v=42"></script>'
            f'<script type="text/javascript">var constellationWidgetUrl{cw2}, '
            f"constellationTypeModule{cw2};"
            f'constellationWidgetUrl{cw2} = "https://reservation.elloha.com/Widget/BookingEngine/{cw1}?idoi={idoi}&culture=fr-FR";'
            f'constellationTypeModule{cw2}=1; constellationWidgetLoad("ConstellationWidgetContainer{cw1}");'
            f'constellationWidgetAddEvent(window, "resize", function () {{constellationWidgetSetAppearance("ConstellationWidgetContainer{cw1}");}});</script>'
        )


class CognitoformPlugin(BasePlugin):
    def __call__(self, parts, config={}):
        if "cognitoforms" in parts.netloc:
            self.parts = parts
            self.config = config
            return self
        return None

    @property
    def contents(self):
        current_lang = self.config["current_lang"]
        extra_params = self.config["extra_params"]

        error_message = _(
            "With a cognitoform plugin, extra params can be void but if you complete it you must specify : scrolling:(yes/no) and overflow:(hidden/scroll/auto)"
        )
        error_message = translate(error_message, target_language=current_lang)
        error_message = (
            f'<div class="cognitoform cognitoform_error">{error_message}</div>'
        )

        if extra_params is None:
            return f'<iframe src="{self.parts.geturl()}" style="border: 0px none; width: {self.config["width"]}%; overflow: auto;" scrolling="yes"></iframe>'
        else:
            if extra_params[0] != "{" or extra_params[-1] != "}":
                return error_message
            try:
                res = json.loads(extra_params.lower())
            except JSONDecodeError:
                return error_message

            if res.get("overflow") is None or res.get("scrolling") is None:
                return error_message

            overflow = res.get("overflow")
            scrolling = res.get("scrolling")
            return f'<iframe src="{self.parts.geturl()}" style="border: 0px none; width: {self.config["width"]}%; overflow: {overflow};" scrolling="{scrolling}"></iframe>'


class ArcgisPlugin(BasePlugin):
    def __call__(self, parts, config={}):
        if "arcgis" in parts.netloc:
            self.parts = parts
            self.config = config
            return self
        return None

    @property
    def contents(self):
        # url : https://developers.arcgis.com/
        # extra_params : {"portal_item_id":"27a432b0835149e6acd3ac39d0e4349c"}
        current_lang = self.config["current_lang"]
        extra_params = self.config["extra_params"]
        url = self.config["url"]

        error_message = _(
            "With arcgis plugin, extra params must contain a dictionary with one key : portal_item_id"
        )
        error_message = translate(error_message, target_language=current_lang)
        error_message = f'<div class="arcgis arcgis_error">{error_message}</div>'

        if extra_params is None or extra_params[0] != "{" or extra_params[-1] != "}":
            return error_message
        try:
            res = json.loads(extra_params.lower())
        except JSONDecodeError:
            return error_message
        if res.get("portal_item_id") is None:
            return error_message

        portal_item_id = res.get("portal_item_id")
        msg = _("Consult the map")
        msg = translate(msg, target_language=current_lang)
        return f'<a href="{url}/view_arcgis?portal_item_id={portal_item_id}">{msg}</a>'


class GiveADayPlugin(BasePlugin):
    def __call__(self, parts, config={}):
        if "www.giveaday.be" in parts.netloc:
            self.parts = parts
            self.config = config
            return self
        #
        return None

    @property
    def contents(self):
        return (
            '<div id="giveaday-widget"></div>'
            "<script src=https://www.giveaday.be/assets/giveaday_v1.js></script>"
            "<script>"
            "function renderWidget() {"
            'giveADayWidget.initialize("giveaday-widget", 2924, "organization", "large", "fr","#34B78F");}'
            "renderWidget();"
            "</script>"
        )


class IIdeluxWastePlugin(Interface):
    """"""


@implementer(IIdeluxWastePlugin)
class IdeluxWastePlugin(BasePlugin):
    def __call__(self, parts, config={}):
        # https://www.odwb.be/api/explore/v2.1/catalog/datasets/guide-de-tri1/records
        if "www.odwb.be" in parts.netloc and "guide-de-tri1" in parts.path:
            self.parts = parts
            self.config = config
            return self
        return None

    # url = config["url"]
    # return (f'<a href="{url}/view_idelux_waste?portal_item_id={portal_item_id}">{msg}</a>')
    @property
    def contents(self):
        headers = {"Accept": "application/json"}
        try:
            # ?limit=-1 to get all records from an odwb table.
            response = requests.get(
                f"{self.parts.geturl()}?limit=-1", headers=headers, timeout=5
            )
            self.config["extra_params"] = response.json()
            return self.config["extra_params"]

        except requests.exceptions.Timeout:
            logger.warning(f"Timeout raised for requests : {self.url}")
            return None


class IInbwContainersAffluencePlugin(Interface):
    """"""


@implementer(IInbwContainersAffluencePlugin)
class InbwContainersAffluencePlugin(BasePlugin):
    def __call__(self, parts, config={}):
        if "www.odwb.be" in parts.netloc and "affluence" in parts.path:
            self.parts = parts
            self.config = config
            return self
        return None

    @property
    def contents(self):
        headers = {"Accept": "application/json"}
        try:
            response = requests.get(
                f"{self.parts.geturl()}&limit=-1", headers=headers, timeout=5
            )
            self.config["extra_params"] = json.dumps(response.json())
            return self.config["extra_params"]
        except requests.exceptions.Timeout:
            logger.warning(f"Timeout raised for requests : {self.url}")
            return None


class IOdwbWidgetPlugin(Interface):
    """"""


@implementer(IOdwbWidgetPlugin)
class OdwbWidgetPlugin(BasePlugin):
    def __call__(self, parts, config={}):
        # https://static.opendatasoft.com/ods-widgets
        if "static.opendatasoft.com" in parts.netloc:
            self.is_odwb_widget_plugins = True
            self.parts = parts
            self.config = config
            return self

    @property
    def contents(self):
        return self.config["extra_params"]


class UnknowServicePlugin(BasePlugin):
    def __call__(self, parts, config={}):
        self.parts = parts
        self.config = config
        return self

    @property
    def contents(self):
        current_lang = self.config["current_lang"]
        msg = _("Unknow service")
        msg = translate(msg, target_language=current_lang)
        return f'<p class="unknow_service">{msg}</p>'


@implementer(IArcgisViewUtils)
class ArcgisView(BrowserView):
    _footer = None

    @property
    def get_portal_item_id(self):
        return self.request.portal_item_id


class IdeluxWasteView(BrowserView):

    def __init__(self, context, request):
        """ """


class InbwContainersAffluenceView(BrowserView):

    def __init__(self, context, request):
        """ """


class IOdwbWidgetLayer(IDefaultBrowserLayer):
    """ """


@implementer(IOdwbViewUtils, IOdwbWidgetLayer)
class OdwbWidgetView(BrowserView):

    def __init__(self, context, request):
        """ """



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import Pages
from zope.interface import implementer


class IPortalPage(IPages):
    """Marker interface and Dexterity Python Schema for PortalPage"""


@implementer(IPortalPage)
class PortalPage(Pages):
    """PortalPage class"""



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.contents import IPage
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import ISection
from imio.smartweb.core.contents import ISectionPostit
from imio.smartweb.core.contents import ISectionText
from imio.smartweb.core.utils import concat_voca_term
from plone import api
from plone.app.contenttypes.behaviors.richtext import IRichText
from plone.app.contenttypes.indexers import SearchableText
from plone.app.contenttypes.indexers import _unicode_save_string_concat
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from Products.CMFPlone.utils import base_hasattr


@indexer(IDexterityContent)
def exclude_from_parent_listing(obj):
    if base_hasattr(obj, "exclude_from_parent_listing"):
        return obj.exclude_from_parent_listing
    return False


@indexer(ISection)
def SearchableText_section(obj):
    terms = []
    if not obj.hide_title:
        # Only index titles that are shown to the visitor
        terms.append(obj.title)
    if base_hasattr(obj, "description") and obj.description:
        # Index descriptions (if any) withould bold
        terms.append(obj.description.replace("**", ""))
    return " ".join(terms)


@indexer(ISectionPostit)
def SearchableText_sectionpostit(obj):
    terms = []
    postits = obj.postits or []
    for p in postits:
        terms.append(p["title"])
        terms.append(p["subtitle"])
        terms.append(p["description"].replace("**", ""))
    result = " ".join(terms)
    result = " ".join((result, SearchableText_section(obj)()))
    return result


@indexer(ISectionText)
def SearchableText_sectiontext(obj):
    """Title is always hidden in text sections, so we only index text field"""
    transforms = api.portal.get_tool("portal_transforms")
    textvalue = IRichText(obj).text
    raw = textvalue.raw
    text = (
        transforms.convertTo("text/plain", raw, mimetype=textvalue.mimeType)
        .getData()
        .strip()
    )
    return text


@indexer(IPages)
def SearchableText_pages(obj):
    """Construct SearchableText of pages with SearchableText of its sections"""
    catalog = api.portal.get_tool("portal_catalog")
    result = _unicode_save_string_concat(SearchableText(obj))
    brains = api.content.find(context=obj, depth=1)
    for brain in brains:
        indexes = catalog.getIndexDataForRID(brain.getRID())
        searchable_text = indexes.get("SearchableText") or []
        result = " ".join((result, " ".join(searchable_text)))
    return result


@indexer(IPages)
def latitude_pages(obj):
    map_sections = obj.listFolderContents(
        contentFilter={"portal_type": "imio.smartweb.SectionMap"}
    )
    if not map_sections:
        raise AttributeError
    geolocation = map_sections[0].geolocation
    if geolocation is None:
        raise AttributeError
    return geolocation.latitude


@indexer(IPages)
def longitude_pages(obj):
    map_sections = obj.listFolderContents(
        contentFilter={"portal_type": "imio.smartweb.SectionMap"}
    )
    if not map_sections:
        raise AttributeError
    geolocation = map_sections[0].geolocation
    if geolocation is None:
        raise AttributeError
    return geolocation.longitude


@indexer(IFolder)
def related_quickaccess(obj):
    if obj.quick_access_items is None:
        return []
    return [item.to_object.UID() for item in obj.quick_access_items]


@indexer(IPage)
def concat_category_topics_indexer(obj):
    category = getattr(obj.aq_base, "page_category", None)
    topics = getattr(obj.aq_base, "topics", [])

    index = []
    if not category and not topics:
        return index
    if topics:
        for topic in topics:
            index.append(concat_voca_term(category, topic))
    else:
        index = category
    return index



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets.httpheaders import HeaderViewlet


class ArcgisHeaderViewlet(HeaderViewlet):
    def update(self):
        super(ArcgisHeaderViewlet, self).update()


class OdwbWidgetHeaderViewlet(HeaderViewlet):

    def should_render(self):
        view = self.view
        if hasattr(view, "should_display_odwb_widget_viewlet"):
            return view.should_display_odwb_widget_viewlet
        return False

    def render(self):
        # if self.should_render():
        return self.index()
        # return ""



# -*- coding: utf-8 -*-

from embeddify import Embedder
from imio.smartweb.core.contents.sections.views import SectionView
from imio.smartweb.core.utils import get_scale_url
from plone.app.contenttypes.utils import replace_link_variables_by_paths


class SlideView(SectionView):
    """Selection Section view"""

    def get_embed_video(self, width=800, height=600):
        embedder = Embedder(width=width, height=height)
        return embedder(self.context.video_url, params=dict(autoplay=True))

    def get_scale_url(self):
        context = self.context
        request = self.request
        return get_scale_url(context, request, "image", "banner")

    def link(self):
        context = self.context
        return replace_link_variables_by_paths(context, context.remoteUrl)



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import DESCRIPTION_MAX_LENGTH
from imio.smartweb.core.contents.blocks.link.fields import LinkField
from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.autoform import directives
from plone.namedfile.field import NamedBlobImage
from zope import schema
from zope.interface import implementer


class ISectionSlide(ISection):
    """Marker interface and Dexterity Python Schema for SectionSlide"""

    directives.omitted(
        "hide_title", "collapsible_section", "background_image", "bootstrap_css_class"
    )

    description = schema.Text(
        title=_("Description"),
        description=_(
            "Use **text** to set text in bold. Limited to ${max} characters.",
            mapping={"max": DESCRIPTION_MAX_LENGTH},
        ),
        max_length=DESCRIPTION_MAX_LENGTH,
        required=False,
    )

    show_title_and_description = schema.Bool(
        title=_(
            "Title and description will be displayed in front of picture as a slogan"
        ),
        required=False,
        default=False,
    )

    image = NamedBlobImage(
        title=_("Image"),
        description=_("Picture will be displayed as slide background"),
        required=False,
    )

    color = schema.TextLine(
        title=_("Color"),
        description=_("Like supplement or alternatively to picture"),
        required=False,
    )

    video_url = schema.URI(
        title=_("Youtube video URL"),
        description=_("Alternatively as a picture"),
        required=False,
    )

    link_title = schema.TextLine(
        title=_("Link title"),
        description=_("Link title will be display on picture as clickable element"),
        required=False,
    )

    remoteUrl = LinkField(title=_("Link URL"), required=False)

    open_in_new_tab = schema.Bool(
        title=_("Open in a new tab"), required=False, default=False
    )


@implementer(ISectionSlide)
class SectionSlide(Section):
    """SectionSlide class"""

    can_toggle_title_visibility = False



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_sections_types
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue
from plone.uuid.interfaces import IUUID
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


class TestIndexes(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="My page",
        )

    def test_SearchableText_section(self):
        catalog = api.portal.get_tool("portal_catalog")
        section_types = get_sections_types()
        for section_type in section_types:
            if section_type == "imio.smartweb.SectionText":
                # see test_SearchableText_sectiontext
                continue
            elif section_type == "imio.smartweb.SectionPostit":
                # see test_SearchableText_sectionpostit
                continue
            title = "Title{}".format(section_type.split(".")[-1])
            section = api.content.create(
                container=self.page,
                type=section_type,
                title=title,
            )
            uuid = IUUID(section)
            brain = api.content.find(UID=uuid)[0]
            indexes = catalog.getIndexDataForRID(brain.getRID())
            self.assertEqual(indexes.get("SearchableText"), [title.lower()])
            section.hide_title = True
            section.reindexObject()
            brain = api.content.find(UID=uuid)[0]
            indexes = catalog.getIndexDataForRID(brain.getRID())
            self.assertEqual(indexes.get("SearchableText"), [])

    def test_SearchableText_section_with_description(self):
        catalog = api.portal.get_tool("portal_catalog")
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionLinks",
            title="Links",
        )
        section.description = "Description **bold**"
        section.reindexObject()
        uuid = IUUID(section)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText"),
            ["links", "description", "bold"],
        )

        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionVideo",
            title="Video",
        )
        section.description = "Description **bold**"
        section.video_url = "https://www.youtube.com/embed/_dOAthafoGQ?feature=oembed"
        section.reindexObject()
        uuid = IUUID(section)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText"),
            ["video", "description", "bold"],
        )

        self.page.reindexObject()
        uuid = IUUID(self.page)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertCountEqual(
            indexes.get("SearchableText"),
            [
                "my",
                "page",
                "my",
                "page",
                "links",
                "description",
                "bold",
                "video",
                "description",
                "bold",
            ],
        )

    def test_SearchableText_sectiontext(self):
        catalog = api.portal.get_tool("portal_catalog")
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionText",
            title="Textsectiontitle",
        )
        uuid = IUUID(section)
        self.assertEqual(len(api.content.find(SearchableText="Textsectiontitle")), 0)
        section.text = RichTextValue("<p>Textsectionbody</p>", "text/html", "text/html")
        section.reindexObject()
        self.assertEqual(len(api.content.find(SearchableText="Textsectionbody")), 1)
        self.page.reindexObject()
        self.assertEqual(len(api.content.find(SearchableText="Textsectionbody")), 2)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("SearchableText"), ["textsectionbody"])

    def test_SearchableText_sectionpostit(self):
        catalog = api.portal.get_tool("portal_catalog")
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionPostit",
            title="Postitsectiontitle",
        )
        uuid = IUUID(section)
        postit1 = {
            "title": "title1",
            "subtitle": "subtitle1",
            "description": "description1 with **bold**",
        }
        postit2 = {
            "title": "title2",
            "subtitle": "subtitle2",
            "description": "description2",
        }
        section.postits = [postit1, postit2]
        section.reindexObject()
        self.assertEqual(len(api.content.find(SearchableText="Postitsectiontitle")), 2)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText"),
            [
                "title1",
                "subtitle1",
                "description1",
                "with",
                "bold",
                "title2",
                "subtitle2",
                "description2",
                "postitsectiontitle",
            ],
        )

    def test_SearchableText_pages(self):
        catalog = api.portal.get_tool("portal_catalog")
        section_types = get_sections_types()
        texts = []
        texts_page = ["my", "page", "my", "page"]  # id & title of page are indexed
        for section_type in section_types:
            title = "Title{}".format(section_type.split(".")[-1])
            section = api.content.create(
                container=self.page,
                type=section_type,
                title=title,
            )
            if section_type == "imio.smartweb.SectionText":
                section.text = RichTextValue(
                    "<p>Textsectionbody</p>", "text/html", "text/html"
                )
                section.reindexObject()
                # section text body is indexed
                texts.append("textsectionbody")
            else:
                # sections titles are indexed by default
                texts.append(title.lower())
        texts += texts_page
        uuid = IUUID(self.page)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertCountEqual(indexes.get("SearchableText"), texts)
        for obj in self.page.listFolderContents():
            api.content.delete(obj)
        brain = api.content.find(UID=uuid)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertCountEqual(indexes.get("SearchableText"), texts_page)

    def test_related_quickaccess(self):
        catalog = api.portal.get_tool("portal_catalog")
        intids = getUtility(IIntIds)
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
        )
        uuid_folder = IUUID(folder)
        page2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page 2",
        )
        folder.quick_access_items = [
            RelationValue(intids.getId(self.page)),
            RelationValue(intids.getId(page2)),
        ]
        folder.reindexObject()
        uuid_qa1 = IUUID(self.page)
        uuid_qa2 = IUUID(page2)
        brain = api.content.find(UID=uuid_folder)[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertCountEqual(indexes.get("related_quickaccess"), [uuid_qa1, uuid_qa2])
        self.assertCountEqual(brain.related_quickaccess, [uuid_qa1, uuid_qa2])

    def test_category_topics_2_topics(self):
        api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="vocabulary test page",
            page_category="emploi",
            topics=["culture", "health"],
        )

        portal_catalog = api.portal.get_tool("portal_catalog")
        result = portal_catalog._catalog.indexes["category_and_topics"]
        indexes = [x for x in result.uniqueValues()]
        self.assertEqual(sorted(indexes), ["emploi-culture", "emploi-health"])

        brains1 = api.content.find(
            category_and_topics="emploi-culture", context=self.portal
        )
        self.assertEqual(brains1[0].Title, "vocabulary test page")

        brains2 = api.content.find(
            category_and_topics="emploi-health", context=self.portal
        )
        self.assertEqual(brains2[0].Title, "vocabulary test page")

    def test_category_topics_1_topics(self):
        api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="vocabulary test page",
            page_category="emploi",
            topics=["mobility"],
        )

        portal_catalog = api.portal.get_tool("portal_catalog")
        result = portal_catalog._catalog.indexes["category_and_topics"]
        indexes = [x for x in result.uniqueValues()]

        self.assertEqual(sorted(indexes), ["emploi-mobility"])

        brains1 = api.content.find(
            category_and_topics="emploi-mobility", context=self.portal
        )
        self.assertEqual(brains1[0].Title, "vocabulary test page")

    def test_category_topics_no_topics(self):
        api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="vocabulary test page",
            page_category="emploi",
        )

        portal_catalog = api.portal.get_tool("portal_catalog")
        result = portal_catalog._catalog.indexes["category_and_topics"]
        indexes = [x for x in result.uniqueValues()]

        self.assertEqual(sorted(indexes), ["emploi"])

        brains1 = api.content.find(category_and_topics="emploi", context=self.portal)
        self.assertEqual(brains1[0].Title, "vocabulary test page")



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.footer import FooterViewlet
from imio.smartweb.core.viewlets.footer import MinisiteFooterViewlet
from imio.smartweb.core.viewlets.footer import SubsiteFooterViewlet
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.testing import logout
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from unittest.mock import patch
from zope.component import getMultiAdapter
from zope.component import getUtility


class TestFooter(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )

    def test_add_footer_to_plonesite(self):
        view = getMultiAdapter((self.portal, self.request), name="footer_settings")
        self.assertTrue(view.available())
        viewlet = FooterViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        view.add_footer()
        footers = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        self.assertEqual(len(footers), 1)
        self.assertFalse(view.available())
        viewlet = FooterViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        self.assertEqual(viewlet.footer, footers[0])
        api.content.delete(footers[0])
        self.assertTrue(view.available())

    def test_add_footer_to_folder(self):
        view = getMultiAdapter((self.folder, self.request), name="footer_settings")
        self.assertFalse(view.available())
        view.add_footer()
        footers = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        self.assertEqual(len(footers), 0)

    def test_add_footer_to_subsite(self):
        footer_view = getMultiAdapter(
            (self.folder, self.request), name="footer_settings"
        )
        self.assertFalse(footer_view.available())
        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        subsite_view.enable()
        self.assertTrue(footer_view.available())
        footer_view.add_footer()
        footers = self.folder.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        self.assertEqual(len(footers), 1)
        self.assertFalse(footer_view.available())
        api.content.delete(footers[0])
        self.assertTrue(footer_view.available())

    def test_add_footer_to_nested_subsite(self):
        nested_folder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Nested folder",
            id="nested_folder",
        )
        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        subsite_view.enable()
        footer_view = getMultiAdapter(
            (self.folder, self.request), name="footer_settings"
        )
        footer_view.add_footer()

        nested_subsite_view = getMultiAdapter(
            (nested_folder, self.request), name="subsite_settings"
        )
        nested_subsite_view.enable()
        nested_footer_view = getMultiAdapter(
            (nested_folder, self.request), name="footer_settings"
        )
        nested_footer_view.add_footer()

        viewlet = SubsiteFooterViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        nested_viewlet = SubsiteFooterViewlet(nested_folder, self.request, None, None)
        nested_viewlet.update()
        self.assertTrue(nested_viewlet.available())
        self.assertNotEqual(viewlet.footer, nested_viewlet.footer)

    def test_add_footer_to_minisite(self):
        footer_view = getMultiAdapter(
            (self.folder, self.request), name="footer_settings"
        )
        self.assertFalse(footer_view.available())
        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        minisite_view.enable()
        self.assertTrue(footer_view.available())
        footer_view.add_footer()
        footers = self.folder.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        self.assertEqual(len(footers), 1)
        self.assertFalse(footer_view.available())
        api.content.delete(footers[0])
        self.assertTrue(footer_view.available())

    def test_portal_and_minisite_footers(self):
        view = getMultiAdapter((self.portal, self.request), name="footer_settings")
        view.add_footer()
        viewlet = FooterViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())
        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        minisite_view.enable()
        viewlet = FooterViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        footer_view = getMultiAdapter(
            (self.folder, self.request), name="footer_settings"
        )
        footer_view.add_footer()
        viewlet = MinisiteFooterViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available())

        footer = getattr(self.folder, "footer")
        api.content.create(
            container=footer,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        viewlet = MinisiteFooterViewlet(self.folder, self.request, None, None)
        viewlet.update()
        sections = viewlet.sections
        self.assertEqual(len(sections), 1)
        self.assertIn("Section text", viewlet.footer())

    def test_exclude_from_parent_listing(self):
        view = getMultiAdapter((self.portal, self.request), name="footer_settings")
        view.add_footer()
        footers = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        self.assertTrue(footers[0].exclude_from_parent_listing)

    def test_background_style(self):
        footer_view = getMultiAdapter(
            (self.portal, self.request), name="footer_settings"
        )
        footer_view.add_footer()
        footer = getattr(self.portal, "footer")
        viewlet = FooterViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.background_style(), "")
        footer.background_image = NamedBlobImage(**make_named_image())
        self.assertIn(
            "background-image:url(http://nohost/plone/footer/@@images/background_image-750",
            viewlet.background_style(),
        )

    def test_sections_render(self):
        footer_view = getMultiAdapter(
            (self.portal, self.request), name="footer_settings"
        )
        footer_view.add_footer()
        footer = getattr(self.portal, "footer")
        api.content.create(
            container=footer,
            type="imio.smartweb.SectionText",
            title="Section text",
        )
        viewlet = FooterViewlet(self.portal, self.request, None, None)
        viewlet.update()
        sections = viewlet.sections
        self.assertEqual(len(sections), 1)
        render = "\n".join(
            [
                getMultiAdapter(
                    (section, self.request), name="full_view_item_without_edit"
                )()
                for section in sections
            ]
        )
        self.assertIn("Section text", render)
        self.assertNotIn('class="manage-section"', render)
        links_section = api.content.create(
            container=footer,
            type="imio.smartweb.SectionLinks",
            title="Section links",
        )
        api.content.create(
            container=links_section,
            type="imio.smartweb.BlockLink",
            title="My link",
        )
        viewlet.update()
        sections = viewlet.sections
        self.assertEqual(len(sections), 2)
        render = "\n".join(
            [
                getMultiAdapter(
                    (section, self.request), name="full_view_item_without_edit"
                )()
                for section in sections
            ]
        )
        self.assertIn("Section links", render)

    def test_has_gdpr_text(self):
        footer_view = getMultiAdapter(
            (self.portal, self.request), name="footer_settings"
        )
        footer_view.add_footer()
        footer = getattr(self.portal, "footer")
        self.assertEqual(footer.restrictedTraverse("@@has_gdpr_text")(), False)
        registry = getUtility(IRegistry)
        records = registry.records
        record = Record(
            field.Bool(
                title="test",
                required=False,
                description="test",
            )
        )
        records["imio.gdpr.interfaces.IGDPRSettings.is_text_ready"] = record
        api.portal.set_registry_record(
            "imio.gdpr.interfaces.IGDPRSettings.is_text_ready", True
        )
        self.assertEqual(footer.restrictedTraverse("@@has_gdpr_text")(), True)

    def test_section_error(self):
        footer_view = getMultiAdapter(
            (self.portal, self.request), name="footer_settings"
        )
        footer_view.add_footer()
        footer = getattr(self.portal, "footer")
        api.content.create(
            container=footer,
            type="imio.smartweb.SectionLinks",
            title="Section links",
        )
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        api.content.transition(page, "publish")
        view = getMultiAdapter((page, self.request), name="full_view")
        self.assertNotIn("Error in section :", view())
        with patch(
            "imio.smartweb.core.contents.sections.links.view.LinksView.items",
            side_effect=Exception,
        ):
            self.assertIn('Error in section : "Section links"', view())
            logout()
            self.assertNotIn("Error in section :", view())



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets.common import (
    ContentViewsViewlet as ContentViewsViewletBase,
)


class ContentViewsViewlet(ContentViewsViewletBase):
    """Add preview action to primary actions to ensure ordering"""

    primary = ["folderContents", "edit", "view", "preview"]



# -*- coding: utf-8 -*-

from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide unwanted profiles from site-creation and quickinstaller."""
        return [
            "imio.smartweb.core:icons-basic",
            "imio.smartweb.core:icons-contenttypes",
            "imio.smartweb.core:last-compilation",
            "imio.smartweb.core:testing",
            "imio.smartweb.core:uninstall",
        ]

    def getNonInstallableProducts(self):
        """Hide unwanted products from site-creation and quickinstaller."""
        return [
            "imio.smartweb.core.upgrades",
        ]


def post_install(context):
    """Post install script"""


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets.social import SocialTagsViewlet as BaseSocialTagsViewlet


class SocialTagsViewlet(BaseSocialTagsViewlet):
    social_image_scale = "paysage_vignette"



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common

import os


class ChatbotViewlet(common.ViewletBase):
    def update(self):
        self.chatbot_url = os.environ.get("CHATBOT_URL", "")
        self.chatbot_id = os.environ.get("CHATBOT_ID", "")
        self.available = self.chatbot_url and self.chatbot_id



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import hash_md5
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.locking.browser.info import LockInfoViewlet
from plone.protect.interfaces import IDisableCSRFProtection
from Products.Five.browser import BrowserView
from zope.annotation.interfaces import IAnnotations
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.lifecycleevent import modified

import json
import pytz


SECTION_ITEMS_HASH_KEY = "sections-items-hash-key"


class ISectionView(Interface):
    """ """


class SectionView(BrowserView):
    """Section view"""

    def __call__(self):
        self.redirect_to_section(self.context.id)

    def locking_info(self):
        return LockInfoViewlet(self.context, self.request, None, None).render()

    def redirect_to_section(self, section_id):
        page = self.context.aq_parent
        url = "{}#section-{}".format(page.absolute_url(), section_id)
        self.request.response.redirect(url)

    def hide_section_title(self):
        if not self.context.can_toggle_title_visibility:
            return
        if self.context.collapsible_section:
            return
        self.context.hide_title = True
        api.portal.show_message(_("Section title has been hidden"), self.request)
        self.redirect_to_section(self.context.id)

    def show_section_title(self):
        if not self.context.can_toggle_title_visibility:
            return
        self.context.hide_title = False
        api.portal.show_message(_("Section title has been shown"), self.request)
        self.redirect_to_section(self.context.id)

    @property
    def display_container_title(self):
        return False

    @property
    def is_anonymous(self):
        return api.user.is_anonymous()


class CarouselOrTableSectionView(SectionView):
    """Section view that can display a carousel"""

    @property
    def image_scale(self):
        layout = self.context.getLayout()
        # scales used depend on the batch size
        if layout == "carousel_view":
            return self.context.nb_results_by_batch == 1 and "affiche" or "vignette"
        elif layout == "table_view":
            return self.context.nb_results_by_batch == 1 and "liste" or "vignette"
        else:
            return getattr(self.context, "image_scale", "")

    def datetime_format(self, item):
        """
        item.get("effective", None)
        => DateTime('YYYY/MM/DD HH:mm:ss.000000 GMT+1')
        convert to more conventional datetime format
        and return its string representation
        """
        effective = item.get("effective", None)
        if effective is None:
            return ""
        target_timezone = pytz.timezone("Europe/Paris")
        if isinstance(effective, str):
            return effective
        dt = effective.asdatetime()
        target_datetime = dt.astimezone(target_timezone)
        output_format = "%Y-%m-%dT%H:%M:%S%z"
        formatted_datetime_str = target_datetime.strftime(output_format)
        return formatted_datetime_str


class HashableJsonSectionView(SectionView):
    json_data = None

    def refresh_modification_date(self):
        new_hash = None
        if self.json_data is not None:
            new_hash = hash_md5(json.dumps(self.json_data))
        annotations = IAnnotations(self.context)
        stored_hash = annotations.get(SECTION_ITEMS_HASH_KEY)
        if stored_hash != new_hash:
            alsoProvides(self.request, IDisableCSRFProtection)
            modified(self.context)
            annotations[SECTION_ITEMS_HASH_KEY] = new_hash



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.multilingual import api as api_lng
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.testing.zope import Browser
from Products.CMFPlone.utils import get_installer

import transaction


class TestRedirectToMainReactView(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        installer = get_installer(self.portal, self.layer["request"])
        installer.install_product("plone.app.multilingual")
        self.rest_directory = api.content.create(
            container=self.portal,
            type="imio.smartweb.DirectoryView",
            title="directory view",
        )
        self.rest_agenda = api.content.create(
            container=self.portal,
            type="imio.smartweb.EventsView",
            title="agenda view",
        )
        self.rest_news = api.content.create(
            container=self.portal,
            type="imio.smartweb.NewsView",
            title="news view",
        )
        api.content.transition(self.rest_directory, "publish")
        api.content.transition(self.rest_agenda, "publish")
        api.content.transition(self.rest_news, "publish")

    def test_rest_view_redirection(self):

        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )
        rest_views = {
            "directory": self.rest_directory,
            "events": self.rest_agenda,
            "news": self.rest_news,
        }
        browser = Browser(self.layer["app"])
        for k, v in rest_views.items():
            api.portal.set_registry_record(f"smartweb.default_{k}_view", v.UID())
            transaction.commit()
            rest_view_url = v.absolute_url()
            browser.open(f"{self.portal_url}/@@{k}_view")
            self.assertEquals(browser.url, rest_view_url)

            v = api.content.move(v, folder)
            transaction.commit()
            rest_view_url = v.absolute_url()
            browser.open(f"{self.portal_url}/@@{k}_view")
            self.assertEquals(browser.url, rest_view_url)

            obj = api_lng.translate(v, target_language="de")
            api.content.transition(obj, "publish")
            transaction.commit()
            browser.open(f"{self.portal_url}/@@{k}_view?language=de")
            self.assertEquals(browser.url, obj.absolute_url())

            api.content.delete(v)
            transaction.commit()
            browser.open(f"{self.portal_url}/@@{k}_view")
            self.assertEquals(browser.url, self.portal_url)



# -*- coding: utf-8 -*-

from Products.Five.browser import BrowserView

import requests


class CirkwiViewView(BrowserView):
    cirwki_api_uri = "https://www.modulesbox.com/fr/api/module/"

    def get_cirkwi_html(self):
        cirkwi_widget_id = self.context.cirkwi_widget_id
        url = f"{self.cirwki_api_uri}{cirkwi_widget_id}"
        response = requests.get(url)
        if response.status_code != 200:
            return response.status_code
        return response.text



# -*- coding: utf-8 -*-

from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets import common
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.utils import parent


class BaseFooterViewlet(common.ViewletBase):
    _footer = None

    def available(self):
        return self.footer is not None

    def background_style(self):
        footer = self.footer
        if not footer.background_image:
            return ""
        scale_url = get_scale_url(
            self.footer, self.request, "background_image", "affiche", "paysage"
        )
        css_bg_image = f"background-image:url({scale_url});"
        css_bg_size = "background-size:cover;"
        return " ".join([css_bg_image, css_bg_size])

    def update(self):
        if not self.available():
            return
        self.sections = self.footer.listFolderContents()


class FooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        root = api.portal.get_navigation_root(self.context)
        if IImioSmartwebMinisite.providedBy(root):
            # don't display portal footer in a minisite
            return
        available_langs = api.portal.get_registry_record("plone.available_languages")
        if IPloneSiteRoot.providedBy(root) and len(available_langs) > 1:
            # multilingual : we should display current lang footer
            current_lang = api.portal.get_current_language()[:2]
            root = getattr(root, current_lang, root)
        footers = root.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        if len(footers) > 0:
            self._footer = footers[0]
            return self._footer


class SubsiteFooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-subsite-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        obj = self.context
        while not INavigationRoot.providedBy(obj):
            if IImioSmartwebSubsite.providedBy(obj):
                footers = obj.listFolderContents(
                    contentFilter={"portal_type": "imio.smartweb.Footer"}
                )
                if len(footers) > 0:
                    self._footer = footers[0]
                    return self._footer
            obj = parent(obj)


class MinisiteFooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-minisite-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        root = api.portal.get_navigation_root(self.context)
        if IImioSmartwebMinisite.providedBy(root):
            footers = root.listFolderContents(
                contentFilter={"portal_type": "imio.smartweb.Footer"}
            )
            if len(footers) > 0:
                self._footer = footers[0]
                return self._footer



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.viewlets.common import escape
from plone.app.layout.viewlets.common import GlobalSectionsViewlet
from plone.memoize.view import memoize
from Products.CMFPlone.utils import safe_unicode
from zope.i18n import translate

import Missing


class ImprovedGlobalSectionsViewlet(GlobalSectionsViewlet):
    _close_menu_markup = (
        '<a aria-label="{back}" class="prev-nav"><span>{back}</span></a>'
        '<a aria-label="{close}" class="close-nav"><span class="close-nav-icon"></span></a>'
    )
    _prev_menu_markup = (
        '<a aria-label="{back}" class="prev-nav"><span>{back}</span></a>'
    )
    _subtree_markup_wrapper = '<ul tabindex="-1">{out}{qa_out}</ul>'
    _submenu_markup_wrapper = '<div class="has_subtree dropdown">{menu_action}<span class="nav-title"><a href="{url}">{title}</span></a>{sub}</div>'
    _quickaccesses_markup_wrapper = '<li class="quick-access"><span class="quick-access-title">{title}</span><ul>{out}</ul></li>'
    _item_markup_template = (
        '<li class="nav_{id}{has_sub_class} nav-item">'
        '<a href="{url}" class="state-{review_state} nav-link"{aria_haspopup}>{title}</a>{opener}'  # noqa: E 501
        "{sub_wrapper}"
        "</li>"
    )

    @property
    @memoize
    def root_depth(self):
        return len(self.nav_root.getPhysicalPath())

    @property
    @memoize
    def nav_root(self):
        return api.portal.get_navigation_root(self.context)

    @property
    @memoize
    def current_lang(self):
        current_lang = api.portal.get_current_language()[:2]
        return current_lang

    def render_quickaccess(self, item):
        return self._item_markup_template.format(**item)

    def render_item(self, item, path):
        sub = self.build_tree(item["path"], first_run=False)
        if sub:
            item.update(
                {
                    "sub": sub,
                    "sub_wrapper": "",
                    "menu_action": "",
                    "opener": self._opener_markup_template.format(**item),
                    "aria_haspopup": ' aria-haspopup="true"',
                    "has_sub_class": " has_subtree",
                }
            )
        else:
            item.update(
                {
                    "sub": sub,
                    "sub_wrapper": "",
                    "menu_action": "",
                    "opener": "",
                    "aria_haspopup": "",
                    "has_sub_class": "",
                }
            )

        if not sub:
            return self._item_markup_template.format(**item)

        level = len(item["path"].split("/")) - self.root_depth
        back_str = translate(_("Back"), target_language=self.current_lang)
        if level == 1:
            # We add "Back" & "Close" buttons on the first dropdown menu level
            close_str = translate(_("Close"), target_language=self.current_lang)
            item["menu_action"] = self._close_menu_markup.format(
                back=back_str, close=close_str
            )
        elif level > 1:
            # We add a "Back" button on the next dropdown menu levels
            item["menu_action"] = self._prev_menu_markup.format(back=back_str)
        item["sub_wrapper"] = self._submenu_markup_wrapper.format(**item)
        return self._item_markup_template.format(**item)

    def build_quickaccess(self, path):
        path_brains = api.content.find(
            path={"query": path, "depth": 0},
        )
        quick_access_uids = path_brains[0].related_quickaccess
        if quick_access_uids == Missing.Value:
            return ""
        quick_access_brains = api.content.find(UID=path_brains[0].related_quickaccess)
        out = ""
        for brain in quick_access_brains:
            entry = {
                "id": brain.getId,
                "path": brain.getPath(),
                "uid": brain.UID,
                "url": brain.getURL(),
                "title": escape(safe_unicode(brain.Title)),
                "review_state": brain.review_state,
                "sub": "",
                "sub_wrapper": "",
                "menu_action": "",
                "opener": "",
                "aria_haspopup": "",
                "has_sub_class": "",
            }
            out += self.render_quickaccess(entry)
        return out

    def remove_items(self, iface, remove_self=False):
        """
        Remove element (based on its interface) children from navigation and
        also remove the element itself, if specified so
        """
        brains = api.content.find(context=self.nav_root, object_provides=iface)
        paths = [b.getPath() for b in brains]
        if self.navtree_path in paths:
            # we never need to remove the root of the menu (ex: subsite menu)
            paths.remove(self.navtree_path)
        to_delete = set()
        for path in paths:
            if path in self.navtree:
                # Item representing the submenu of the matched element
                to_delete.add(path)
            for key in self.navtree.keys():
                if key.startswith("{}/".format(path)):
                    # Item representing a (grand)children of the matched element
                    to_delete.add(key)
            if remove_self:
                parent_path = "/".join(path.rsplit("/")[:-1])
                parent_navtree = self.navtree[parent_path]
                for i in range(len(parent_navtree)):
                    if parent_navtree[i]["path"] == path:
                        # Item representing the matched element
                        del parent_navtree[i]
                        break
        for key in to_delete:
            del self.navtree[key]

    def remove_minisites(self):
        """We need to remove minisites from navigation, only in main portal"""
        if not IImioSmartwebMinisite.providedBy(self.nav_root):
            self.remove_items(IImioSmartwebMinisite, remove_self=True)

    def remove_subsites_children(self):
        """We need to remove subsites children from navigation"""
        self.remove_items(IImioSmartwebSubsite)

    def build_tree(self, path, first_run=True):
        """We add quick access contents to the standard Plone navigation"""
        out = ""
        for item in self.navtree.get(path, []):
            out += self.render_item(item, path)

        if not first_run and out:
            # Quick accesses are displayed on every levels of the menu
            qa_menu = self.build_quickaccess(path)
            qa_out = ""
            if qa_menu:
                qa_out = self._quickaccesses_markup_wrapper.format(
                    title=translate(
                        _("Quick access"), target_language=self.current_lang
                    ),
                    out=qa_menu,
                )
            out = self._subtree_markup_wrapper.format(out=out, qa_out=qa_out)
        return out

    def render_globalnav(self):
        self.remove_minisites()
        self.remove_subsites_children()
        return self.build_tree(self.navtree_path)



# -*- coding: utf-8 -*-

from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform import directives
from plone.z3cform import layout
from zope import schema
from zope.interface import Interface


class ISendinblueTextRowSchema(Interface):
    language = schema.TextLine(
        title=_("Language (en, fr,...)"),
        description=_("Enter the language code. Ex.: en"),
    )

    text = schema.TextLine(title=_("Text"), description=_("Your button title"))


class ISmartwebControlPanel(Interface):
    url_formdefs_api = schema.TextLine(
        title=_("Url to get forms from your e-guichet"),
        description=_(
            "Example : https://COMMUNE-formulaires.guichet-citoyen.be/api/formdefs/"
        ),
        required=False,
    )

    secret_key_api = schema.Password(
        title=_("Secret key"),
        description=_("Secret key to use API"),
        required=False,
    )

    propose_directory_url = schema.URI(
        title=_("Url to propose a new citizen contact"),
        required=False,
    )

    propose_events_url = schema.URI(
        title=_("Url to propose a new citizen event"),
        required=False,
    )

    propose_news_url = schema.URI(
        title=_("Url to propose a new citizen news"),
        required=False,
    )

    directory_entity_uid = schema.Choice(
        title=_("Directory entity"),
        source="imio.smartweb.vocabulary.RemoteDirectoryEntities",
        required=False,
    )

    events_entity_uid = schema.Choice(
        title=_("Events entity"),
        source="imio.smartweb.vocabulary.RemoteEventsEntities",
        required=False,
    )

    news_entity_uid = schema.Choice(
        title=_("News entity"),
        source="imio.smartweb.vocabulary.RemoteNewsEntities",
        required=False,
    )

    directory_solr_core = schema.TextLine(
        title=_("Directory SolR Core ID"),
        default="directory",
        required=False,
    )

    events_solr_core = schema.TextLine(
        title=_("Events SolR Core ID"),
        default="events",
        required=False,
    )

    news_solr_core = schema.TextLine(
        title=_("News SolR Core ID"),
        default="news",
        required=False,
    )

    default_directory_view = schema.Choice(
        title=_("Default directory view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.DirectoryViews",
        required=False,
    )

    default_events_view = schema.Choice(
        title=_("Default events view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.EventsViews",
        required=False,
    )

    default_news_view = schema.Choice(
        title=_("Default news view"),
        description=_("This information is used for search results redirection"),
        source="imio.smartweb.vocabulary.NewsViews",
        required=False,
    )

    category_and_topics_vocabulary = schema.Dict(
        title=_("Category and topics vocabulary setting for TeleServices"),
        description=_(
            "Choose a term from the vocabulary and define the corresponding name"
        ),
        key_type=schema.Choice(
            title=_("Vocabulary term"),
            source="imio.smartweb.vocabulary.CategoryAndTopics",
        ),
        value_type=schema.TextLine(
            title=_("Name"),
        ),
        required=False,
    )

    sendinblue_button_position = schema.Choice(
        title=_("SendInBlue : Define button position"),
        source="imio.smartweb.vocabulary.SendInBlueButtonPosition",
        default="button_bottom",
        required=True,
    )

    sendinblue_button_text = schema.List(
        title=_("SendInBlue : Define button text"),
        description=_("Choose SendInblue submission button text"),
        value_type=DictRow(
            title="Value",
            schema=ISendinblueTextRowSchema,
        ),
        default=[],
        required=True,
    )
    directives.widget("sendinblue_button_text", DataGridFieldFactory, auto_append=False)

    plausible_url = schema.TextLine(
        title=_("Plausible URL"),
        description=_(
            "Example : plausible.imio.be (SMARTWEB_PLAUSIBLE_URL varenv has precedence over this.)"
        ),
        required=False,
    )

    plausible_site = schema.TextLine(
        title=_("Plausible Site"),
        description=_(
            "Example : namur.be (SMARTWEB_PLAUSIBLE_SITE varenv has precedence over this.)"
        ),
        required=False,
    )

    plausible_token = schema.TextLine(
        title=_("Plausible Token"),
        description=_(
            "Plausible authentification token (SMARTWEB_PLAUSIBLE_TOKEN varenv has precedence over this.)"
        ),
        required=False,
    )


class SmartwebControlPanelForm(RegistryEditForm):
    schema = ISmartwebControlPanel
    schema_prefix = "smartweb"
    label = _("Smartweb Settings")


SmartwebControlPanelView = layout.wrap_form(
    SmartwebControlPanelForm, ControlPanelFormWrapper
)



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.zope import Browser
from zope.component import getMultiAdapter

import transaction


class TestLocalRoles(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Page",
        )

    def test_local_manager_in_sharing(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertNotIn("Can manage locally", content)

        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        transaction.commit()
        browser.open("{}/@@sharing".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertIn("Can manage locally", content)

    def test_local_manager_for_minisite_logo(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        setRoles(
            self.portal, TEST_USER_ID, ["Reader", "Editor", "Contributor", "Reviewer"]
        )
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertNotIn(
            '"formfield-form-widgets-IImioSmartwebMinisiteSettings-logo"', content
        )

        self.folder.manage_setLocalRoles(TEST_USER_ID, ("Local Manager",))
        transaction.commit()
        browser.open("{}/edit".format(self.folder.absolute_url()))
        content = browser.contents
        self.assertIn(
            '"formfield-form-widgets-IImioSmartwebMinisiteSettings-logo"', content
        )

    def test_local_manager_for_new_html_section(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        setRoles(
            self.portal, TEST_USER_ID, ["Reader", "Editor", "Contributor", "Reviewer"]
        )
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}".format(self.page.absolute_url()))
        content = browser.contents
        self.assertNotIn("++add++imio.smartweb.SectionHTML", content)

        self.folder.manage_setLocalRoles(TEST_USER_ID, ("Local Manager",))
        transaction.commit()
        browser.open("{}".format(self.page.absolute_url()))
        content = browser.contents
        self.assertIn("++add++imio.smartweb.SectionHTML", content)

    def test_local_manager_for_existing_html_section(self):
        section = api.content.create(
            container=self.page,
            type="imio.smartweb.SectionHTML",
            title="Section",
        )
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        setRoles(
            self.portal, TEST_USER_ID, ["Reader", "Editor", "Contributor", "Reviewer"]
        )
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(section.absolute_url()))
        content = browser.contents
        self.assertNotIn("formfield-form-widgets-html", content)

        self.folder.manage_setLocalRoles(TEST_USER_ID, ("Local Manager",))
        transaction.commit()
        browser.open("{}/edit".format(section.absolute_url()))
        content = browser.contents
        self.assertIn("formfield-form-widgets-html", content)



# -*- coding: utf-8 -*-

from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserSubMenuItem


class IAuthenticSourcesMenu(IBrowserMenu):
    """The authentic sources menu."""


class ISmartwebHelpMenu(IBrowserMenu):
    """The smartweb help menu."""


class IAuthenticSourcesSubMenuItem(IBrowserSubMenuItem):
    """The authentic sources submenu item."""


class ISmartwebHelpSubMenuItem(IBrowserSubMenuItem):
    """The smartweb help submenu item."""



# -*- coding: utf-8 -*-
from imio.smartweb.core.contents import IProcedure
from imio.smartweb.core.contents.pages.views import PagesView
from imio.smartweb.core.tests.utils import get_json
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.publisher.browser import TestRequest
from zope.viewlet.interfaces import IViewletManager

import json
import requests_mock

GUICHET_URL = "https://demo.guichet-citoyen.be/api/formdefs/"


class TestProcedure(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def _changeUser(self, loginName):
        logout()
        login(self.portal, loginName)
        self.member = api.user.get_current()
        self.request["AUTHENTICATED_USER"] = self.member

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            "imio.smartweb.Folder",
            self.portal,
            "parent_container",
            title="Parent container",
        )
        self.parent = self.portal[parent_id]
        self._changeUser("test")
        self.json_procedures_raw_mock = get_json(
            "resources/json_procedures_raw_mock.json"
        )

    def test_ct_procedure_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Procedure")
        schema = fti.lookupSchema()
        self.assertEqual(IProcedure, schema)

    def test_ct_procedure_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Procedure")
        self.assertTrue(fti)

    def test_ct_procedure_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Procedure")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IProcedure.providedBy(obj),
            "IProcedure not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_procedure_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.parent,
            type="imio.smartweb.Procedure",
            id="procedure",
        )

        self.assertTrue(
            IProcedure.providedBy(obj),
            "IProcedure not provided by {0}!".format(
                obj.id,
            ),
        )

        parent = obj.__parent__
        self.assertIn("procedure", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("procedure", parent.objectIds())

    def test_ct_procedure_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.Procedure")
        self.assertTrue(fti.global_allow, "{0} is globally addable!".format(fti.id))

    @requests_mock.Mocker()
    def test_procedure_invariant(self, m):
        m.get(GUICHET_URL, text=json.dumps(self.json_procedures_raw_mock))
        request = TestRequest(
            form={
                "form.widgets.IBasic.title": "My Procedure",
                "form.widgets.procedure_url": "http://another_e_guichet.be/procedure1",
            }
        )
        alsoProvides(request, IPloneFormLayer)
        form = DefaultAddForm(self.portal, request)
        form.portal_type = "imio.smartweb.Procedure"
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 0)

        request = TestRequest(
            form={
                "form.widgets.IBasic.title": "My Procedure",
            }
        )
        alsoProvides(request, IPloneFormLayer)
        form = DefaultAddForm(self.portal, request)
        form.portal_type = "imio.smartweb.Procedure"
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 0)

        request = TestRequest(
            form={
                "form.widgets.IBasic.title": "My Procedure",
                "form.widgets.procedure_url": "http://another_e_guichet.be/procedure1",
                "form.widgets.procedure_ts": "https://demo-formulaires.guichet-citoyen.be/acte-de-deces/",
            }
        )
        alsoProvides(request, IPloneFormLayer)
        form = DefaultAddForm(self.portal, request)
        form.portal_type = "imio.smartweb.Procedure"
        form.update()
        data, errors = form.extractData()
        self.assertTrue(len(errors) == 1)
        self.assertIn(errors[0].message, "Only one procedure field can be filled !")

    @requests_mock.Mocker()
    def test_procedure_viewlet(self, m):
        m.get(GUICHET_URL, text=json.dumps(self.json_procedures_raw_mock))
        procedure = api.content.create(
            container=self.portal,
            type="imio.smartweb.Procedure",
            id="procedure",
        )

        view = PagesView(self.portal["procedure"], self.request)
        manager_name = "plone.abovecontentbody"

        manager = queryMultiAdapter(
            (self.portal["procedure"], self.request, view),
            IViewletManager,
            manager_name,
            default=None,
        )
        manager.update()
        viewlet = [
            v for v in manager.viewlets if v.__name__ == "imio.smartweb.procedure"
        ]
        self.assertEqual(len(viewlet), 1)
        self.assertEqual("", viewlet[0]().strip())
        url = "http://another_e_guichet.be/procedure1"
        procedure.procedure_url = url
        self.assertIn(url, viewlet[0]())
        procedure.procedure_url = None
        procedure.procedure_ts = (
            "https://demo-formulaires.guichet-citoyen.be/acte-de-deces/"
        )
        self.assertIn("Acte de d\\u00e9c\\u00e8s", viewlet[0]())



# -*- coding: utf-8 -*-

from io import BytesIO
from lxml import etree
from Products.CMFPlone.browser.icons import IconsView
from Products.CMFPlone.browser.icons import SVG_MODIFER
from zExceptions import NotFound
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import logging

logger = logging.getLogger("imio.smartweb.core")


@implementer(IPublishTraverse)
class SmartwebIconsView(IconsView):
    prefix = "smartweb.icon."

    def _iconfile(self, icon):
        site = getSite()
        try:
            # Use unrestrictedTraverse to handle icons stored in portal_resources
            return site.unrestrictedTraverse(icon)
        except NotFound:
            logger.exception(
                f"Icon resolver lookup of '{icon}' failed, fallback to Plone icon."
            )
            return site.restrictedTraverse(self.defaulticon)

    def tag(self, name, tag_class="", tag_alt=""):
        try:
            return super(SmartwebIconsView, self).tag(name, tag_class, tag_alt)
        except NotImplementedError:
            # Resolving icon stored in database, let's do it below
            # See https://github.com/plone/Products.CMFPlone/pull/3359
            pass
        icon = self.lookup(name)
        iconfile = self._iconfile(icon)
        fh = BytesIO(iconfile.data)
        try:
            svgtree = etree.parse(fh)
        except etree.XMLSyntaxError:
            logger.exception(f"SVG File from Database: {iconfile.absolute_url()}")
            return iconfile.data

        if svgtree.docinfo.root_name.lower() != "svg":
            raise ValueError(
                f"SVG file content root tag mismatch (not svg but {svgtree.docinfo.root_name}): {iconfile.path}"
            )
        modifier_cfg = {
            "cssclass": tag_class,
            "title": tag_alt,
        }
        for name, modifier in SVG_MODIFER.items():
            __traceback_info__ = name
            modifier(svgtree, modifier_cfg)
        return etree.tostring(svgtree)



# -*- coding: utf-8 -*-

from eea.facetednavigation.config import ANNO_FACETED_LAYOUT
from eea.facetednavigation.layout.layout import FacetedLayout
from zope.annotation.interfaces import IAnnotations


class SmartwebFacetedLayout(FacetedLayout):
    @property
    def layout(self):
        return IAnnotations(self.context).get(
            ANNO_FACETED_LAYOUT, "faceted-summary-view"
        )



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IContextAwareDefaultFactory


@provider(IContextAwareDefaultFactory)
def see_all_default(context):
    return translate(_("See all"), context=getRequest())


class ISectionCollection(ISection):
    """Marker interface and Dexterity Python Schema for SectionCollection"""

    directives.widget(
        "collection",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Collection"],
            "favorites": [],
        },
    )
    collection = RelationChoice(
        title=_("Select a collection"),
        vocabulary="plone.app.vocabularies.Catalog",
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    max_nb_batches = schema.Int(
        title=_("Maximum number of batches to display"),
        required=True,
        default=2,
        min=1,
        max=12,
    )

    link_text = schema.TextLine(
        title=_("Text for the link to the collection"),
        defaultFactory=see_all_default,
        required=True,
    )

    model.fieldset(
        "layout",
        fields=[
            "show_items_lead_image",
            "show_items_description",
            "show_items_date",
        ],
    )
    show_items_lead_image = schema.Bool(
        title=_("Show items lead image"), required=False
    )

    show_items_description = schema.Bool(
        title=_("Show items description"), required=False
    )

    show_items_date = schema.Bool(
        title=_("Show items publication date"), required=False
    )


@implementer(ISectionCollection)
class SectionCollection(Section):
    """SectionCollection class"""

    manage_display = True



# -*- coding: utf-8 -*-
from plone.app.content.browser.vocabulary import VocabularyView
from plone.app.content.browser.vocabulary import VocabLookupException
from plone.app.content.utils import json_dumps


class SmartwebVocabularyView(VocabularyView):

    def __call__(self):
        form = self.request.form
        name = form.get("name")
        if name != "imio.smartweb.vocabulary.RemoteContacts":
            return super(SmartwebVocabularyView, self).__call__()

        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )

        try:
            vocabulary = self.get_vocabulary()
        except VocabLookupException as e:
            return json_dumps({"error": e.args[0]})

        query = form.get("query")
        items = []
        if not query:
            items = vocabulary
        else:
            for term in vocabulary:
                if query.lower() in term.title.lower():
                    items.append(term)

        results = [{"id": item.value, "text": item.title} for item in items]

        return json_dumps({"results": results, "total": len(results)})



# # -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.config import EVENTS_URL
from imio.smartweb.core.config import NEWS_URL
from imio.smartweb.core.utils import get_wca_token
from plone.memoize import ram
from plone.memoize.view import memoize
from time import time
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import json
import os
import requests


def get_entity_id(vocabulary_name, uid):
    vocabulary = getUtility(IVocabularyFactory, name=vocabulary_name)()
    for term in vocabulary:
        if term.token == uid:
            return term.title or term.id
    return None


def get_auth_sources_response(
    sources_name, entity_id, wf_status=["published"], time_to_cache=60
):
    """_summary_

    Args:
        sources_name (_type_): 3 values are possible: "directory", "news", "events"
        entity_id (_type_): _description_

    Returns:
        _type_: json response
    """
    url = ""
    if sources_name == "directory":
        url = DIRECTORY_URL
        portaltype = "imio.directory.Contact"
        client_id_varname = "RESTAPI_DIRECTORY_CLIENT_ID"
        client_secret_varname = "RESTAPI_DIRECTORY_CLIENT_SECRET"
    elif sources_name == "events":
        url = EVENTS_URL
        portaltype = "imio.events.Event"
        client_id_varname = "RESTAPI_EVENTS_CLIENT_ID"
        client_secret_varname = "RESTAPI_EVENTS_CLIENT_SECRET"
    elif sources_name == "news":
        url = NEWS_URL
        portaltype = "imio.news.NewsItem"
        client_id_varname = "RESTAPI_NEWS_CLIENT_ID"
        client_secret_varname = "RESTAPI_NEWS_CLIENT_SECRET"

    payload = {
        "query": [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": [portaltype],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.absolutePath",
                "v": f"/{entity_id}",
            },
            {
                "i": "review_state",
                "o": "plone.app.querystring.operation.selection.any",
                "v": wf_status,
            },
        ],
        "metadata_fields": ["title", "modified"],
        "sort_on": "effective",
        "sort_order": "descending",
        "fullobjects": True,
        "b_start": 0,
        "b_size": 4000,
    }
    client_id = os.environ.get(client_id_varname)
    client_secret = os.environ.get(client_secret_varname)
    auth = get_wca_token(client_id, client_secret)
    headers = {
        "Accept": "application/json",
        "Content-Type": "text/plain",
        "Authorization": auth,
    }
    url = f"{url}/@querystring-search"
    response = get_cached_data(url, headers, payload, time_to_cache)
    return response


def _cache_key(func, url, headers, payload, time_to_cache):
    return (url, time() // time_to_cache)


@ram.cache(_cache_key)
def get_cached_data(url, headers, payload, time_to_cache):
    return requests.request("POST", url, headers=headers, data=json.dumps(payload))



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import IPortalPage
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.utils import get_default_content_id
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides


class TestPortalPage(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.authorized_types_in_portal_page = [
            "imio.smartweb.SectionContact",
            "imio.smartweb.SectionFiles",
            "imio.smartweb.SectionGallery",
            "imio.smartweb.SectionLinks",
            "imio.smartweb.SectionSelections",
            "imio.smartweb.SectionText",
            "imio.smartweb.SectionVideo",
        ]
        self.unauthorized_types_in_portal_page = [
            "Document",
            "Link",
            "File",
            "Image",
            "imio.smartweb.BlockLink",
            "imio.smartweb.DirectoryView",
            "imio.smartweb.EventsView",
            "imio.smartweb.Folder",
            "imio.smartweb.Footer",
            "imio.smartweb.NewsView",
            "imio.smartweb.Page",
            "imio.smartweb.PortalPage",
            "imio.smartweb.Procedure",
        ]

        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            id="folder",
        )

    def test_ct_portal_page_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.PortalPage")
        schema = fti.lookupSchema()
        self.assertEqual(IPortalPage, schema)

    def test_ct_portal_page_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.PortalPage")
        self.assertTrue(fti)

    def test_ct_portal_page_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.PortalPage")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IPortalPage.providedBy(obj),
            "IPortalPage not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_portal_page_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        portal_page = api.content.create(
            container=self.folder,
            type="imio.smartweb.PortalPage",
            title="portal_page",
        )
        self.assertTrue(
            IPortalPage.providedBy(portal_page),
            "IPortalPage not provided by {0}!".format(
                portal_page.id,
            ),
        )
        parent = portal_page.__parent__
        self.assertIn("portal_page", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=portal_page)
        self.assertNotIn("portal_page", parent.objectIds())

    def test_ct_portal_page_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.smartweb.PortalPage")
        self.assertTrue(fti.global_allow, "{0} is not globally addable!".format(fti.id))

    def test_ct_portal_page_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        portal_page = api.content.create(
            container=self.folder,
            type="imio.smartweb.PortalPage",
            title="portal_page",
        )
        for t in self.unauthorized_types_in_portal_page:
            with self.assertRaises(InvalidParameterError):
                api.content.create(
                    container=portal_page,
                    type=t,
                    title="My {}".format(t),
                )
        for t in self.authorized_types_in_portal_page:
            api.content.create(
                container=portal_page,
                type=t,
                title="My {}".format(t),
            )

    def test_js_bundles(self):
        portal_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="portal_page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)

        getMultiAdapter((portal_page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 0)

        api.content.create(
            container=portal_page,
            type="imio.smartweb.SectionGallery",
            title="Gallery",
        )
        getMultiAdapter((portal_page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 2)
        self.assertListEqual(bundles, ["spotlight", "flexbin"])

    def test_js_bundles_with_contact(self):
        portal_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="portal_page",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)

        getMultiAdapter((portal_page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 0)

        section_contact = api.content.create(
            container=portal_page,
            type="imio.smartweb.SectionContact",
            title="Contact",
        )
        getMultiAdapter((portal_page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 2)
        self.assertListEqual(bundles, ["spotlight", "flexbin"])

        section_contact.gallery_mode = "swiper"
        setattr(self.request, "enabled_bundles", [])
        getMultiAdapter((portal_page, self.request), name="full_view")()
        bundles = getattr(self.request, "enabled_bundles", [])
        self.assertEqual(len(bundles), 1)
        self.assertListEqual(bundles, ["spotlight"])

    def test_no_title(self):
        portal_page = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            title="My Portal Page",
        )
        view = getMultiAdapter((portal_page, self.request), name="full_view")()
        self.assertNotIn("<h1>", view)

    def test_get_default_content_id(self):
        self.assertEqual(get_default_content_id(self.portal), "")
        api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            id="portal-page",
        )
        self.portal.setDefaultPage("portal-page")
        self.assertEqual(get_default_content_id(self.portal), "portal-page")



# -*- coding: utf-8 -*-

from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_INTEGRATION_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.category import CategoryViewlet
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID


class TestCategories(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_viewlet_on_content_with_no_category(self):
        viewlet = CategoryViewlet(self.portal, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        self.assertIsNone(viewlet.get_category())

    def test_viewlet_on_page(self):
        page = api.content.create(
            container=self.portal,
            type="imio.smartweb.Page",
            title="Page",
        )
        viewlet = CategoryViewlet(page, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        self.assertIsNone(viewlet.get_category())
        page.taxonomy_page_category = "publication"
        self.assertTrue(viewlet.available())
        self.assertEqual(viewlet.get_category(), "Publication")

    def test_viewlet_on_procedure(self):
        procedure = api.content.create(
            container=self.portal,
            type="imio.smartweb.Procedure",
            title="Procedure",
        )
        viewlet = CategoryViewlet(procedure, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        self.assertIsNone(viewlet.get_category())
        procedure.taxonomy_procedure_category = "autorisation_carte"
        self.assertTrue(viewlet.available())
        self.assertEqual(viewlet.get_category(), "Authorization and card")



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.viewlets.navigation import ImprovedGlobalSectionsViewlet
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets import common
from plone.memoize.view import memoize
from Products.CMFPlone.browser.navigation import CatalogNavigationTabs
from Products.CMFPlone.utils import parent
from zope.component import getMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class NavigationSettingsProxy(object):
    generate_tabs = None
    navigation_depth = None
    displayed_types = None
    sort_tabs_on = None
    sort_tabs_reversed = None
    nonfolderish_tabs = None
    filter_on_workflow = None
    workflow_states_to_show = None
    show_excluded_items = None


class SubsiteNavigationTabs(CatalogNavigationTabs):
    navtree_path = None

    def _getNavQuery(self):
        query = super(SubsiteNavigationTabs, self)._getNavQuery()
        query["path"]["query"] = self.navtree_path
        return query


class SubsiteHeaderViewlet(common.ViewletBase):
    """Viewlet containing the subsite header viewlets manager"""


class SubsiteHeaderViewletsManager(IViewletManager):
    """Viewlet manager containing subsite header viewlets"""


class BaseSubsiteViewlet(common.ViewletBase):
    _subsite_root = None

    @property
    def subsite_root(self):
        if self._subsite_root is not None:
            return self._subsite_root
        obj = self.context
        while not IImioSmartwebSubsite.providedBy(
            obj
        ) and not INavigationRoot.providedBy(obj):
            obj = parent(obj)
        if IImioSmartwebSubsite.providedBy(obj):
            self._subsite_root = obj
            return self._subsite_root

    def available(self):
        return self.subsite_root is not None


class SubsiteNavigationViewlet(BaseSubsiteViewlet, ImprovedGlobalSectionsViewlet):
    @property
    def settings(self):
        settings = super(SubsiteNavigationViewlet, self).settings
        mutable_settings = NavigationSettingsProxy()
        mutable_settings.generate_tabs = settings.generate_tabs
        mutable_settings.displayed_types = settings.displayed_types
        mutable_settings.sort_tabs_on = settings.sort_tabs_on
        mutable_settings.sort_tabs_reversed = settings.sort_tabs_reversed
        mutable_settings.nonfolderish_tabs = settings.nonfolderish_tabs
        mutable_settings.filter_on_workflow = settings.filter_on_workflow
        mutable_settings.workflow_states_to_show = settings.workflow_states_to_show
        mutable_settings.show_excluded_items = settings.show_excluded_items
        # we need to change navigation_depth (but we cannot change registry
        # value), so we use our own settings object proxy.
        # This is caused by navtree_depth property deprecation.
        mutable_settings.navigation_depth = self.subsite_root.menu_depth
        return mutable_settings

    @property
    def navtree_path(self):
        return "/".join(self.subsite_root.getPhysicalPath())

    @property
    def nav_root(self):
        return self.subsite_root

    @property
    @memoize
    def root_depth(self):
        return len(self.subsite_root.getPhysicalPath())

    @property
    @memoize
    def portal_tabs(self):
        subsite_tabs_view = getMultiAdapter(
            (self.context, self.request), name="subsite_tabs_view"
        )
        subsite_tabs_view.navtree_path = self.navtree_path
        return subsite_tabs_view.topLevelTabs(actions=[])

    def render_globalnav(self):
        self.remove_subsites_children()
        return self.build_tree(self.navtree_path)


class SubsiteLogoViewlet(BaseSubsiteViewlet):
    def show_logo(self):
        if self.subsite_root.logo is None:
            return False
        return self.subsite_root.logo_display_mode in ["logo", "logo_title"]

    def show_title(self):
        return self.subsite_root.logo_display_mode in ["title", "logo_title"]

    def get_logo_scale_url(self):
        request = self.request
        return get_scale_url(self.subsite_root, request, "logo", "preview")



# -*- coding: utf-8 -*-

from imio.smartweb.core.browser.banner.settings import ILocallyHiddenBanner
from imio.smartweb.core.utils import hash_md5
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets import common
from plone.namedfile.file import NamedBlobImage
from Products.CMFPlone.utils import base_hasattr


class BannerViewlet(common.ViewletBase):
    _banner_item = None
    _banner_is_hidden = False

    def available(self):
        if self.banner_item is None:
            # There is no banner defined in parents
            return False
        elif self.is_banner_hidden and not self.is_banner_locally_hidden:
            # Banner is hidden in a parent content
            return False
        can_edit = api.user.has_permission("Modify portal content", obj=self.context)
        if self.is_banner_locally_hidden and not can_edit:
            # Banner is hidden locally, but user can not edit its display
            return False
        return True

    @property
    def banner_item(self):
        if self._banner_item is not None:
            return self._banner_item
        for item in self.context.aq_chain:
            if ILocallyHiddenBanner.providedBy(item):
                self._banner_is_hidden = True
            if (
                base_hasattr(item, "banner")
                and item.banner is not None
                and isinstance(item.banner, NamedBlobImage)
            ):
                self._banner_item = item
                return item
            if INavigationRoot.providedBy(item):
                return

    @property
    def is_banner_hidden(self):
        return self.banner_item and self._banner_is_hidden

    @property
    def is_banner_locally_hidden(self):
        return ILocallyHiddenBanner.providedBy(self.context)

    def background_style(self):
        if self.is_banner_hidden:
            return ""
        modification_date = self.banner_item.ModificationDate()
        modified_hash = hash_md5(modification_date)
        image_url = f"{self.banner_item.absolute_url()}/@@images/banner?cache_key={modified_hash}"
        css_bg_image = f"background-image:url({image_url});"
        css_bg_size = "background-size:cover;"
        return " ".join([css_bg_image, css_bg_size])



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IContextAwareDefaultFactory
from zope.interface import provider


@provider(IContextAwareDefaultFactory)
def see_all_default(context):
    return translate(_("See all events"), context=getRequest())


class ISectionEvents(ISection):
    """Marker interface and Dexterity Python Schema for SectionEvents"""

    directives.widget(related_events=SelectFieldWidget)
    related_events = schema.Choice(
        title=_("Related agenda"),
        source="imio.smartweb.vocabulary.RemoteAgendas",
        required=True,
    )

    specific_related_events = schema.List(
        title=_("Specific related events"),
        description=_("Get priory on related agenda for these specifics events."),
        value_type=schema.Choice(source="imio.smartweb.vocabulary.EventsFromEntity"),
        required=False,
    )
    directives.widget(
        "specific_related_events",
        AjaxSelectFieldWidget,
        source="imio.smartweb.vocabulary.EventsFromEntity",
    )

    directives.widget(
        "linking_rest_view",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["imio.smartweb.EventsView"],
            "favorites": [],
        },
    )
    linking_rest_view = RelationChoice(
        title=_("Events view used to display events details"),
        vocabulary="plone.app.vocabularies.Catalog",
        required=True,
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    max_nb_batches = schema.Int(
        title=_("Maximum number of batches to display"),
        required=True,
        default=2,
        min=1,
        max=12,
    )

    link_text = schema.TextLine(
        title=_("Text for the link to the events view"),
        defaultFactory=see_all_default,
        required=True,
    )

    model.fieldset("layout", fields=["show_items_description"])
    show_items_description = schema.Bool(
        title=_("Show items description"), required=False
    )

    model.fieldset("layout", fields=["display_agendas_titles"])
    display_agendas_titles = schema.Bool(
        title=_("Display agendas titles"),
        description=_("If checked, display agendas titles in the carousel view."),
        required=False,
    )


@implementer(ISectionEvents)
class SectionEvents(Section):
    """SectionEvents class"""

    manage_display = True
    show_items_date = True



# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from imio.smartweb.core.utils import get_default_content_id
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets import common


class HeroBannerViewlet(common.ViewletBase):
    _herobanner = None

    def available(self):
        return self.herobanner is not None

    def update(self):
        if not self.available():
            return
        # we need to tell the template that we are rendering for the viewlet
        self.request.set("viewlet_rendering", True)
        self.sections = self.herobanner.listFolderContents()

    @property
    def herobanner(self):
        if self._herobanner is not None:
            return self._herobanner

        obj = self.context
        if not INavigationRoot.providedBy(obj):
            parent = aq_parent(obj)
            if not INavigationRoot.providedBy(parent):
                return

            default_page_id = get_default_content_id(parent)
            if default_page_id == obj.id:
                obj = parent
            else:
                return

        herobanners = obj.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.HeroBanner"}
        )
        if len(herobanners) > 0:
            self._herobanner = herobanners[0]
            return self._herobanner



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup

from collective.messagesviewlet.message import add_timezone
from collective.messagesviewlet.utils import add_message
from datetime import datetime
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.viewlets.logo import LogoViewlet
from imio.smartweb.core.viewlets.messages import MessagesViewlet
from imio.smartweb.core.viewlets.minisite import MinisitePortalLinkViewlet
from imio.smartweb.core.viewlets.navigation import ImprovedGlobalSectionsViewlet
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.dexterity.content import ASSIGNABLE_CACHE_KEY
from plone.testing.zope import Browser
from plone.namedfile.file import NamedBlobImage
from unittest import mock
from unittest.mock import patch
from z3c.relationfield import RelationValue
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import transaction


class TestMinisite(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        self.message_config_folder = self.portal["messages-config"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        # avoid cached empty value for instance behaviors
        delattr(self.request, ASSIGNABLE_CACHE_KEY)

    def _clean_cache(self):
        # utils.get_messages_to_show is cached, remove infos in request annotation
        cache_keys = [
            k
            for k in IAnnotations(self.request)
            if k.startswith("messagesviewlet-utils-get_messages_to_show-")
        ]
        for cache_key in cache_keys:
            del IAnnotations(self.request)[cache_key]

    def test_activation(self):
        view = getMultiAdapter((self.portal, self.request), name="minisite_settings")
        self.assertFalse(view.available())

        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        self.assertTrue(view.available())
        self.assertFalse(view.enabled())

        view.enable()
        self.assertTrue(IImioSmartwebMinisite.providedBy(self.folder))
        self.assertTrue(INavigationRoot.providedBy(self.folder))

        self.assertFalse(view.available())
        self.assertTrue(view.enabled())

        view.disable()
        self.assertFalse(IImioSmartwebMinisite.providedBy(self.folder))
        self.assertFalse(INavigationRoot.providedBy(self.folder))
        self.assertTrue(view.available())
        self.assertFalse(view.enabled())

        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        subsite_view.enable()
        self.assertFalse(view.available())

        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        view = getMultiAdapter((subfolder, self.request), name="minisite_settings")
        self.assertFalse(view.available())
        view.enable()
        self.assertFalse(view.enabled())

    def test_minisite_exclude_from_nav(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebMinisite.providedBy(self.folder))
        self.assertTrue(self.folder.exclude_from_nav)

        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open("{}/edit".format(self.folder.absolute_url()))
        content = browser.contents
        soup = BeautifulSoup(content)
        exclude_from_nav_widget = soup.find(
            id="form-widgets-IExcludeFromNavigation-exclude_from_nav"
        )
        self.assertIsNone(exclude_from_nav_widget.find("input"))
        children = [
            c for c in exclude_from_nav_widget if not isinstance(c, str) or c.strip()
        ]
        self.assertEqual(len(children), 1)
        self.assertEqual(children[0].text, "yes")

    def test_minisite_navigation(self):
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
            id="subfolder",
        )
        api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Subpage",
            id="subpage",
        )
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        viewlet = ImprovedGlobalSectionsViewlet(self.folder, self.request, None, None)
        viewlet.update()
        viewlet.remove_minisites()
        self.assertEqual(len(viewlet.navtree), 2)
        self.assertEqual(len(viewlet.navtree["/plone/folder"]), 2)
        self.assertIn("subfolder", viewlet.render_globalnav())
        self.assertIn("subpage", viewlet.render_globalnav())

    def test_quick_accesses(self):
        api.portal.set_registry_record("plone.navigation_depth", 5)
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder level 2",
        )
        api.content.create(
            container=subfolder,
            type="imio.smartweb.Folder",
            title="Subfolder level 3 (to display submenu)",
        )
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Quick Page everywhere",
        )
        page_sub = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Quick Page Sub",
        )
        viewlet = ImprovedGlobalSectionsViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertNotIn('<li class="quick-access">', viewlet.render_globalnav())

        # Quick access on minisite folder
        intids = getUtility(IIntIds)
        self.folder.quick_access_items = [
            RelationValue(intids.getId(page)),
        ]
        self.folder.reindexObject()
        html = viewlet.render_globalnav()
        self.assertNotIn('<li class="quick-access">', viewlet.render_globalnav())

        # Quick access on second level folder
        intids = getUtility(IIntIds)
        subfolder.quick_access_items = [
            RelationValue(intids.getId(page)),
            RelationValue(intids.getId(page_sub)),
        ]
        subfolder.reindexObject()
        html = viewlet.render_globalnav()
        self.assertIn('<li class="quick-access">', viewlet.render_globalnav())

        soup = BeautifulSoup(html)
        qa = soup.find("li", {"class": "nav_subfolder-level-2"}).find(
            "li", {"class": "quick-access"}
        )

        self.assertEqual(len(qa.find_all("li")), 2)
        self.assertIsNotNone(qa.find("li", {"class": "nav_quick-page-sub"}))

    def test_delete_minisite(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        api.content.delete(self.folder)

    def test_move_minisite_in_folder(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        folder2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder2",
            id="folder2",
        )
        moved_folder = api.content.move(self.folder, folder2)
        view = getMultiAdapter((moved_folder, self.request), name="minisite_settings")
        self.assertFalse(view.available())
        self.assertFalse(view.enabled())
        self.assertFalse(IImioSmartwebMinisite.providedBy(moved_folder))

    def test_copy_minisite_in_folder(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        folder2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder2",
            id="folder2",
        )
        copied_folder = api.content.copy(self.folder, folder2)
        view = getMultiAdapter((copied_folder, self.request), name="minisite_settings")
        self.assertFalse(view.available())
        self.assertFalse(view.enabled())
        self.assertFalse(IImioSmartwebMinisite.providedBy(copied_folder))

    def test_move_minisite_in_minisite(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        folder2 = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder2",
            id="folder2",
        )
        view = getMultiAdapter((folder2, self.request), name="minisite_settings")
        view.enable()
        moved_folder = api.content.move(self.folder, folder2)
        view = getMultiAdapter((moved_folder, self.request), name="minisite_settings")
        self.assertFalse(view.available())
        self.assertFalse(view.enabled())
        self.assertFalse(IImioSmartwebMinisite.providedBy(moved_folder))

    def test_minisite_in_minisite(self):
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        self.assertTrue(IImioSmartwebMinisite.providedBy(self.folder))

        minisite2 = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="minisite2",
            id="minisite2",
        )
        view = getMultiAdapter((minisite2, self.request), name="minisite_settings")
        self.assertFalse(view.available())

    def test_cannot_enable_minisite_on_subsite(self):
        subsite_view = getMultiAdapter(
            (self.folder, self.request), name="subsite_settings"
        )
        subsite_view.enable()
        minisite_view = getMultiAdapter(
            (self.folder, self.request), name="minisite_settings"
        )
        self.assertFalse(minisite_view.available())
        self.assertFalse(minisite_view.enabled())
        minisite_view.enable()
        self.assertFalse(minisite_view.enabled())

    def test_minisite_viewlet_logo(self):
        viewlet = LogoViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.show_title)
        self.assertTrue(viewlet.show_logo)
        self.assertFalse(viewlet.is_svg)
        self.assertEqual(viewlet.navigation_root_url, "http://nohost/plone")
        html = viewlet.render()
        soup = BeautifulSoup(html)
        img = soup.find("img")
        self.assertEqual(
            img.get("src"), "http://nohost/plone/++resource++plone-logo.svg"
        )
        annotations = IAnnotations(self.request)
        del annotations["plone.memoize"]
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        viewlet = LogoViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.navigation_root_url, "http://nohost/plone/folder")
        self.assertTrue(viewlet.show_title)
        self.assertFalse(viewlet.show_logo)
        self.folder.logo_display_mode = "logo"
        viewlet.update()
        self.assertFalse(viewlet.show_title)
        self.assertFalse(viewlet.show_logo)
        self.folder.logo_display_mode = "logo_title"
        viewlet.update()
        self.assertTrue(viewlet.show_title)
        self.assertFalse(viewlet.show_logo)
        self.folder.logo = NamedBlobImage(**make_named_image("plone.svg"))
        viewlet.update()
        self.assertTrue(viewlet.show_logo)
        self.assertTrue(viewlet.is_svg)
        self.assertIn(b"<svg", viewlet.svg_data)
        self.folder.logo = NamedBlobImage(**make_named_image("plone.png"))
        viewlet = LogoViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.show_logo)
        self.assertFalse(viewlet.is_svg)
        self.folder.logo_display_mode = "logo"
        viewlet.update()
        self.assertTrue(viewlet.show_logo)
        html = viewlet.render()
        soup = BeautifulSoup(html)
        img = soup.find("img")
        self.assertIn("http://nohost/plone/folder/@@images/logo-400", img.get("src"))

        # Title should remain the same on sub-contents
        subfolder = api.content.create(
            container=self.folder,
            type="imio.smartweb.Folder",
            title="Subfolder",
        )
        viewlet = LogoViewlet(subfolder, self.request, None, None)
        viewlet.update()
        self.assertEqual(viewlet.navigation_root_url, "http://nohost/plone/folder")

    def test_minisite_body_class(self):
        page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Subpage",
            id="subpage",
        )
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        template = self.folder.restrictedTraverse("view")
        layout_view = page.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("is-in-minisite", body_class)

        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.disable()
        layout_view = page.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(template, view)
        self.assertNotIn("is-in-minisite", body_class)

    def test_portal_link_viewlet(self):
        viewlet = MinisitePortalLinkViewlet(self.folder, self.request, None, None)
        viewlet.update()
        self.assertFalse(viewlet.available())
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        self.assertTrue(viewlet.available())
        attr = {"absolute_url.return_value": "http://www.test.be/folder/minisite"}
        with patch(
            "plone.api.portal.get_navigation_root",
            return_value=mock.Mock(**attr),
        ):
            self.assertEqual(viewlet.get_hostname(), "test.be")
        attr = {"absolute_url.return_value": "http://test.be/folder/minisite"}
        with patch(
            "plone.api.portal.get_navigation_root",
            return_value=mock.Mock(**attr),
        ):
            self.assertEqual(viewlet.get_hostname(), "test.be")
        attr = {"absolute_url.return_value": "http://sub.test.be/folder/minisite"}
        with patch(
            "plone.api.portal.get_navigation_root",
            return_value=mock.Mock(**attr),
        ):
            self.assertEqual(viewlet.get_hostname(), "sub.test.be")

    def test_messagesviewlet_in_minisite(self):
        message = add_message(
            "msg",
            "My message title",
            "My message text",
            start=add_timezone(datetime(2019, 10, 26, 12, 0)),
        )
        viewlet = MessagesViewlet(self.folder, self.portal.REQUEST, None, None)
        viewlet.update()
        # no message in viewlet because the message is in "inactive" state
        self.assertEqual(len(viewlet.getAllMessages()), 0)
        api.content.transition(message, "activate")
        message.reindexObject()
        self._clean_cache()
        self.assertEqual(len(viewlet.getAllMessages()), 1)

        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        self.assertTrue(view.available())
        self.assertFalse(view.enabled())
        view.enable()
        self.assertTrue(view.enabled())
        self.assertEqual(len(viewlet.getAllMessages()), 0)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.base import ISection
from imio.smartweb.core.contents.sections.base import Section
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.z3cform.widget import AjaxSelectFieldWidget
from plone.autoform import directives
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import implementer


class ISectionContact(ISection):
    """Marker interface and Dexterity Python Schema for SectionContact"""

    directives.widget(
        "related_contacts",
        AjaxSelectFieldWidget,
        vocabulary="imio.smartweb.vocabulary.RemoteContacts",
        pattern_options={"multiple": True},
    )
    related_contacts = schema.List(
        title=_("Related contacts"),
        description=_(
            "Select contacts. If you can't find contacts you want, make sure "
            """it exists in the directory and that its "state" is published."""
        ),
        value_type=schema.Choice(source="imio.smartweb.vocabulary.RemoteContacts"),
        required=True,
    )

    directives.widget(visible_blocks=CheckBoxFieldWidget)
    visible_blocks = schema.List(
        title=_("Visible blocks"),
        description=_("Blocks that will be displayed in contact"),
        value_type=schema.Choice(vocabulary="imio.smartweb.vocabulary.ContactBlocks"),
        default=["address", "itinerary", "contact_informations", "schedule"],
    )

    model.fieldset(
        "layout",
        fields=[
            "gallery_mode",
            "nb_results_by_batch",
            "image_scale",
        ],
    )

    gallery_mode = schema.Choice(
        title=_("Gallery mode"),
        description=_("Choose your gallery layout mode"),
        source="imio.smartweb.vocabulary.GalleryMode",
        default="gallery",
    )

    nb_results_by_batch = schema.Choice(
        title=_("Number of items per batch (only for carousel mode)"),
        required=True,
        default=3,
        values=[1, 2, 3, 4],
    )

    nb_contact_by_line = schema.Choice(
        title=_("Maximum number of contacts by line"),
        description=_("Maximum number of contacts by line (on PC)"),
        required=True,
        default=4,
        values=[1, 2, 3, 4],
    )

    image_scale = schema.Choice(
        title=_("Image scale for images (only for gallery mode)"),
        default="affiche",
        vocabulary="imio.smartweb.vocabulary.Scales",
        required=True,
    )


@implementer(ISectionContact)
class SectionContact(Section):
    """SectionContact class"""

    can_toggle_title_visibility = False



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import CarouselOrTableSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_scale_url


class LinksView(CarouselOrTableSectionView):
    """Links Section view"""

    def items(self):
        orientation = self.context.orientation
        image_scale = self.image_scale
        items = self.context.listFolderContents()
        results = []
        for item in items:
            url = item.absolute_url()
            has_icon = has_image = False
            if getattr(item.aq_base, "svg_icon", None):
                has_icon = True
            elif getattr(item.aq_base, "image", None):
                has_image = True
            scale_url = get_scale_url(
                item, self.request, "image", image_scale, orientation
            )
            results.append(
                {
                    "title": item.title,
                    "description": item.description,
                    "url": url,
                    "icon": item.svg_icon,
                    "has_icon": has_icon,
                    "image": scale_url,
                    "has_image": has_image,
                    "open_in_new_tab": item.open_in_new_tab,
                }
            )
        return batch_results(results, self.context.nb_results_by_batch)



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.forms import CustomEditForm
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.contents import IDefaultPages
from plone.z3cform import layout
from z3c.form.interfaces import DISPLAY_MODE


class SmartwebCustomEditForm(CustomEditForm):
    def updateFields(self):
        super(SmartwebCustomEditForm, self).updateFields()
        if IDefaultPages.providedBy(self.context) or IImioSmartwebMinisite.providedBy(
            self.context
        ):
            # We need to make exclude_from_nav field read only as it is used
            # for default pages to exclude them from sitemap / navigation / ...
            # and for minisites to exclude them as well.
            for group in self.groups:
                if "IExcludeFromNavigation.exclude_from_nav" in group.fields:
                    group.fields["IExcludeFromNavigation.exclude_from_nav"].mode = (
                        DISPLAY_MODE
                    )


SmartwebCustomEditView = layout.wrap_form(SmartwebCustomEditForm)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.sections.views import CarouselOrTableSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_scale_url
from zope.component import queryMultiAdapter


class FilesView(CarouselOrTableSectionView):
    """Files Section view"""

    def items(self):
        image_scale = self.image_scale
        items = self.context.listFolderContents()
        results = []
        for item in items:
            url = item.absolute_url()
            has_image = True if getattr(item.aq_base, "image", None) else False
            file_view = queryMultiAdapter((item, self.request), name="file_view")
            scale_url = get_scale_url(
                item, self.request, "image", image_scale, orientation="paysage"
            )
            results.append(
                {
                    "title": item.title,
                    "description": item.description,
                    "url": url,
                    "image": scale_url,
                    "has_image": has_image,
                    "item_infos": file_view.human_readable_size(),
                }
            )
        return batch_results(results, self.context.nb_results_by_batch)



# -*- coding: utf-8

from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.intid.interfaces import IIntIds


class TestBehaviors(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def _changeUser(self, loginName):
        logout()
        login(self.portal, loginName)
        self.member = api.user.get_current()
        self.request["AUTHENTICATED_USER"] = self.member

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self._changeUser("test")

    def test_summary_listing(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
        )
        api.content.create(
            container=folder,
            type="imio.smartweb.Folder",
            title="Sub folder",
            id="subfolder",
        )
        page1 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 2",
            id="page2",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        view = getMultiAdapter((folder, self.request), name="summary_view")
        results = view.results()
        self.assertEqual(len(results), 3)

        # a page can be excluded from parent listing
        page1.exclude_from_parent_listing = True
        page1.reindexObject()
        results = view.results()
        self.assertEqual(len(results), 2)

        # other content types are not excluded from parent listing
        api.content.create(
            container=folder,
            type="Link",
            title="Link",
            id="link",
        )
        results = view.results()
        self.assertEqual(len(results), 3)

    def test_block_listing(self):
        folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
        )
        subfolder = api.content.create(
            container=folder,
            type="imio.smartweb.Folder",
            title="Sub folder",
            id="subfolder",
        )
        page1 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 1",
            id="page1",
        )
        page2 = api.content.create(
            container=folder,
            type="imio.smartweb.Page",
            title="Page 2",
            id="page2",
        )
        sub_page = api.content.create(
            container=subfolder,
            type="imio.smartweb.Page",
            title="Sub page",
            id="subpage",
        )
        alsoProvides(self.request, IImioSmartwebCoreLayer)
        view = getMultiAdapter((folder, self.request), name="block_view")
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 3)
        self.assertEqual(len(results["quick_access"]), 0)

        # a sub page can be included in quick access
        intids = getUtility(IIntIds)
        folder.quick_access_items = [
            RelationValue(intids.getId(sub_page)),
        ]
        # sub_page.include_in_quick_access = True
        # sub_page.reindexObject()
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 3)
        self.assertEqual(len(results["quick_access"]), 1)

        # a page can be excluded from parent listing
        page1.exclude_from_parent_listing = True
        page1.reindexObject()
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 2)
        self.assertEqual(len(results["quick_access"]), 1)

        # an excluded page can still be included in quick access
        folder.quick_access_items = [
            RelationValue(intids.getId(sub_page)),
            RelationValue(intids.getId(page1)),
        ]
        # page1.include_in_quick_access = True
        # page1.reindexObject()
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 2)
        self.assertEqual(len(results["quick_access"]), 2)

        results_ids = [b.id for b in results["results"]]
        quick_access_ids = [b.id for b in results["quick_access"]]
        self.assertListEqual(results_ids, ["subfolder", "page2"])
        self.assertListEqual(quick_access_ids, ["page1", "subpage"])

        # the same page cannot be included in quick access and in parent listing
        folder.quick_access_items = [
            RelationValue(intids.getId(sub_page)),
            RelationValue(intids.getId(page1)),
            RelationValue(intids.getId(page2)),
        ]
        # page2.include_in_quick_access = True
        # page2.reindexObject()
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 2)
        self.assertEqual(len(results["quick_access"]), 2)
        results_ids = [b.id for b in results["results"]]
        quick_access_ids = [b.id for b in results["quick_access"]]
        self.assertIn("page2", results_ids)
        self.assertNotIn("page2", quick_access_ids)

        # other content types are not excluded from parent listing
        api.content.create(
            container=folder,
            type="Link",
            title="Link",
            id="link",
        )
        results = view.blocks_results()
        self.assertEqual(len(results["results"]), 3)



# -*- coding: utf-8 -*-

from imio.smartweb.core import config
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.core.utils import get_json
from imio.smartweb.core.viewlets.interfaces import IAuthenticSourcesMenu
from imio.smartweb.core.viewlets.interfaces import IAuthenticSourcesSubMenuItem
from imio.smartweb.core.viewlets.interfaces import ISmartwebHelpMenu
from imio.smartweb.core.viewlets.interfaces import ISmartwebHelpSubMenuItem
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.contentmenu.menu import BrowserMenu
from plone.app.contentmenu.menu import BrowserSubMenuItem
from plone.memoize import ram
from time import time
from zope.interface import implementer

import os
import re


def _cache_key(func, obj, context, request):
    return (obj.id, time() // (5 * 60))


@implementer(IAuthenticSourcesSubMenuItem)
class AuthenticSourcesMenuItem(BrowserSubMenuItem):
    title = _("Authentic sources")
    submenuId = "plone_authentic_sources_menu"
    icon = "boxes"
    extra = {
        "id": "plone-authentic-sources-menu",
        "li_class": "plonetoolbar-authentic-sources-menu",
    }

    order = 50

    @property
    def action(self):
        return "#"

    def available(self):
        if not IImioSmartwebCoreLayer.providedBy(self.request):
            return False
        permission = "Modify portal content"
        if api.user.has_permission(permission, obj=self.context) is False:
            return False
        news_entity_url = api.portal.get_registry_record(
            "smartweb.news_entity_uid", default=None
        )
        events_entity_url = api.portal.get_registry_record(
            "smartweb.events_entity_uid", default=None
        )
        directory_entity_url = api.portal.get_registry_record(
            "smartweb.directory_entity_uid", default=None
        )
        return news_entity_url or events_entity_url or directory_entity_url

    def selected(self):
        return False


@implementer(IAuthenticSourcesMenu)
class AuthenticSourcesMenu(BrowserMenu):
    @ram.cache(_cache_key)
    def getMenuItems(self, context, request):
        news_entity_url = self.get_entity_from_authentic_source(
            config.NEWS_URL, "smartweb.news_entity_uid"
        )
        events_entity_url = self.get_entity_from_authentic_source(
            config.EVENTS_URL, "smartweb.events_entity_uid"
        )
        directory_entity_url = self.get_entity_from_authentic_source(
            config.DIRECTORY_URL, "smartweb.directory_entity_uid"
        )
        news = {"url": news_entity_url, "icon": "newspaper", "id": "news"}
        events = {"url": events_entity_url, "icon": "calendar-event", "id": "events"}
        directory = {
            "url": directory_entity_url,
            "icon": "file-person",
            "id": "directory",
        }
        authentic_sources = [news, events, directory]

        menu_items = []
        for source in authentic_sources:
            if source.get("url") is None:
                continue
            title = re.compile(r"https?://(www\.)?")
            title = title.sub("", source["url"]).strip().strip("/")
            menu_item = {
                "title": title,
                "description": "",
                "action": source["url"],
                "selected": False,
                "icon": source["icon"],
                "extra": {
                    "id": f'plone-authentic-sources-menu{source["id"]}',
                    "separator": None,
                    "class": "",
                },
                "submenu": None,
            }
            menu_items.append(menu_item)
        return menu_items

    def get_entity_from_authentic_source(self, authentic_sources_url, registry_key):
        value = api.portal.get_registry_record(registry_key)
        if value is None:
            return None
        result = get_json(f"{authentic_sources_url}/@search?UID={value}")
        if not result or not result.get("items"):
            return None
        return result.get("items")[0].get("@id")


@implementer(ISmartwebHelpSubMenuItem)
class SmartwebHelpMenuItem(BrowserSubMenuItem):
    title = _("Please help!")
    submenuId = "plone_smartweb_help_menu"
    icon = "info-circle"
    extra = {
        "id": "plone-smartweb-help-menu",
        "li_class": "plonetoolbar-smartweb-help-menu",
    }

    order = 50

    @property
    def action(self):
        return "#"

    def available(self):
        if not IImioSmartwebCoreLayer.providedBy(self.request):
            return False
        permission = "Modify portal content"
        return api.user.has_permission(permission, obj=self.context)


@implementer(ISmartwebHelpMenu)
class SmartwebHelpMenu(BrowserMenu):
    @ram.cache(_cache_key)
    def getMenuItems(self, context, request):
        rtfm_action = os.environ.get(
            "help_menu_rtfm", "https://docs.imio.be/iasmartweb/smartweb_v6"
        )
        support_action = os.environ.get("help_menu_support", "https://support.imio.be")
        workshop_action = os.environ.get(
            "help_menu_workshop",
            "https://my-formulaires.imio.be/ateliers-imio/ateliers-iasmartweb-1",
        )

        # Read The Funny Manual... (no?)
        rtfm = {
            "title": _("Read documentation"),
            "description": "",
            "action": rtfm_action,
            "selected": False,
            "icon": "eye-fill",
            "extra": {
                "id": "plone-smartweb-help-menu-rtfm",
                "separator": None,
                "class": "",
            },
            "submenu": None,
        }

        support = {
            "title": _("Ask a question"),
            "description": "",
            "action": support_action,
            "selected": False,
            "icon": "chat-square-heart-fill",
            "extra": {
                "id": "plone-smartweb-help-menu-support",
                "separator": None,
                "class": "",
            },
            "submenu": None,
        }

        workshop = {
            "title": _("Take part of workshop"),
            "description": "",
            "action": workshop_action,
            "selected": False,
            "icon": "rocket-takeoff-fill",
            "extra": {
                "id": "plone-smartweb-help-menu-workshop",
                "separator": None,
                "class": "",
            },
            "submenu": None,
        }

        menu_items = [rtfm, support, workshop]
        return menu_items



# -*- coding: utf-8 -*-

from datetime import datetime
from freezegun import freeze_time
from imio.smartweb.core.contents.sections.views import SECTION_ITEMS_HASH_KEY
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_json
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from time import sleep
from z3c.relationfield import RelationValue
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.intid.interfaces import IIntIds

import json
import requests_mock


class TestSectionEvents(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portalpage = api.content.create(
            container=self.portal,
            type="imio.smartweb.PortalPage",
            id="Portal page",
        )
        self.json_events = get_json("resources/json_rest_events.json")
        self.json_specific_event = get_json("resources/json_rest_specific_events.json")
        self.rest_events_view = api.content.create(
            container=self.portal,
            type="imio.smartweb.EventsView",
            title="Rest events view",
        )
        self.events = api.content.create(
            container=self.portalpage,
            type="imio.smartweb.SectionEvents",
            title="My events",
        )

    @requests_mock.Mocker()
    @freeze_time("2021-11-15")
    def test_events(self, m):
        intids = getUtility(IIntIds)
        self.events.related_events = "e73e6a81afea4a579cd0da2773af8d29"
        self.events.linking_rest_view = RelationValue(
            intids.getId(self.rest_events_view)
        )
        view = queryMultiAdapter((self.portalpage, self.request), name="full_view")
        self.assertIn("My events", view())
        events_view = queryMultiAdapter(
            (self.events, self.request), name="carousel_view"
        )
        self.assertEqual(events_view.items, [])
        url = "http://localhost:8080/Plone/@events?selected_agendas=e73e6a81afea4a579cd0da2773af8d29&metadata_fields=category_title&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=UID&event_dates.query=2021-11-15&event_dates.range=min&b_size=6&translated_in_en=1&sort_on=event_dates"
        m.get(url, text=json.dumps(self.json_events))
        self.assertEqual(events_view.items[0][0].get("title"), "Journée de l'ATL")
        self.assertEqual(len(events_view.items[0]), 2)
        self.events.specific_related_events = ["1178188bddde4ced95a6cf8bf04c443c"]
        url = "http://localhost:8080/Plone/@events?UID=1178188bddde4ced95a6cf8bf04c443c&metadata_fields=category_title&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=UID&event_dates.query=2021-11-15&event_dates.range=min&b_size=6&translated_in_en=1"
        m.get(url, text=json.dumps(self.json_specific_event))
        self.assertEqual(len(events_view.items[0]), 1)
        self.assertEqual(events_view.items[0][0].get("title"), "Bonne cheville")

    @requests_mock.Mocker()
    def test_events_modified(self, m):
        today = datetime.now()
        today_str = today.strftime("%Y-%m-%d")
        intids = getUtility(IIntIds)
        self.events.related_events = "e73e6a81afea4a579cd0da2773af8d29"
        self.events.linking_rest_view = RelationValue(
            intids.getId(self.rest_events_view)
        )
        annotations = IAnnotations(self.events)
        self.assertIsNone(annotations.get(SECTION_ITEMS_HASH_KEY))

        events_view = queryMultiAdapter(
            (self.events, self.request), name="carousel_view"
        )
        url = f"http://localhost:8080/Plone/@events?selected_agendas=e73e6a81afea4a579cd0da2773af8d29&metadata_fields=category_title&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=UID&event_dates.query={today_str}&event_dates.range=min&b_size=6&translated_in_en=1&sort_on=event_dates"
        m.get(url, text=json.dumps(self.json_events))
        self.assertEqual(len(events_view.items[0]), 2)
        hash_1 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertIsNotNone(hash_1)
        first_modification = self.portalpage.ModificationDate()

        sleep(1)
        url = f"http://localhost:8080/Plone/@events?selected_agendas=e73e6a81afea4a579cd0da2773af8d29&metadata_fields=category_title&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=UID&event_dates.query={today_str}&event_dates.range=min&b_size=6&translated_in_en=1&sort_on=event_dates"
        m.get(url, text="{}")
        self.assertEqual(len(events_view.items), 0)
        next_modification = self.portalpage.ModificationDate()
        hash_2 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertNotEqual(hash_1, hash_2)
        self.assertNotEqual(first_modification, next_modification)

        sleep(1)
        self.assertEqual(len(events_view.items), 0)
        last_modification = self.portalpage.ModificationDate()
        hash_3 = annotations.get(SECTION_ITEMS_HASH_KEY)
        self.assertEqual(hash_2, hash_3)
        self.assertEqual(next_modification, last_modification)

    @requests_mock.Mocker()
    def test_orientation(self, m):
        today = datetime.now()
        today_str = today.strftime("%Y-%m-%d")
        intids = getUtility(IIntIds)
        self.events.related_events = "e73e6a81afea4a579cd0da2773af8d29"
        self.events.linking_rest_view = RelationValue(
            intids.getId(self.rest_events_view)
        )
        events_view = queryMultiAdapter(
            (self.events, self.request), name="carousel_view"
        )
        url = f"http://localhost:8080/Plone/@events?selected_agendas=e73e6a81afea4a579cd0da2773af8d29&metadata_fields=category_title&metadata_fields=start&metadata_fields=end&metadata_fields=has_leadimage&metadata_fields=modified&metadata_fields=UID&event_dates.query={today_str}&event_dates.range=min&b_size=6&translated_in_en=1&sort_on=event_dates"
        m.get(url, text=json.dumps(self.json_events))

        self.assertIn("paysage_vignette", events_view.items[0][0]["image"])
        self.events.orientation = "portrait"
        self.assertIn("portrait_vignette", events_view.items[0][0]["image"])



# -*- coding: utf-8 -*-

from imio.smartweb.core.config import DIRECTORY_URL
from imio.smartweb.core.contents.sections.contact.utils import ContactProperties
from imio.smartweb.core.contents.sections.views import HashableJsonSectionView
from imio.smartweb.core.utils import batch_results
from imio.smartweb.core.utils import get_json
from plone import api


class ContactView(HashableJsonSectionView):
    """Contact Section view"""

    def contacts(self):
        if self.context.related_contacts is None:
            return
        related_contacts = self.context.related_contacts
        uids = "&UID=".join(related_contacts)
        url = "{}/@search?UID={}&fullobjects=1".format(DIRECTORY_URL, uids)
        current_lang = api.portal.get_current_language()[:2]
        if current_lang != "fr":
            url = f"{url}&translated_in_{current_lang}=1"
        self.json_data = get_json(url)
        self.refresh_modification_date()
        if self.json_data is None or len(self.json_data.get("items", [])) == 0:  # NOQA
            return
        results = self.json_data.get("items")
        index_map = {value: index for index, value in enumerate(related_contacts)}
        results = sorted(results, key=lambda x: index_map[x["UID"]])
        return batch_results(results, self.context.nb_contact_by_line)

    def get_contact_properties(self, json_dict):
        return ContactProperties(json_dict, self.context)



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents import RestView
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class INewsView(model.Schema):
    """Marker interface and Dexterity Python Schema for NewsView"""

    selected_news_folder = schema.Choice(
        title=_("Selected news folder"),
        description=_("Select news folder to display"),
        vocabulary="imio.smartweb.vocabulary.RemoteNewsFolders",
        required=True,
    )

    nb_results = schema.Int(
        title=_("Number of items to display"), default=20, required=True
    )

    display_newsfolders_titles = schema.Bool(
        title=_("Display news folders titles"),
        description=_("If checked, display news folders titles"),
        required=False,
    )


@implementer(INewsView)
class NewsView(RestView):
    """NewsView class"""
